/***************************************************************************
 *
 * This file is provided under a dual BSD/GPLv2 license.  When using or 
 *   redistributing this file, you may do so under either license.
 * 
 *   GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007-2013 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *   BSD LICENSE 
 * 
 *   Copyright(c) 2007-2013 Intel Corporation. All rights reserved.
 *   All rights reserved.
 * 
 *   Redistribution and use in source and binary forms, with or without 
 *   modification, are permitted provided that the following conditions 
 *   are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided with the 
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *  version: QAT1.5.L.1.11.0-36
 *
 ***************************************************************************/

/**
 *****************************************************************************
 * @file lac_sym_drbg_ht_test_vectors.h
 *
 * @ingroup LacSym_DrbgHT
 *
 * @description
 *      This file contains definitions of vectors used for DRBG health testing
 *
 *****************************************************************************/
#ifndef LAC_SYM_DRBG_HT_TEST_VECTORS_H
#define LAC_SYM_DRBG_HT_TEST_VECTORS_H

/* Vector definition for Instantiate function health tests */
typedef struct lac_drbg_ht_instantiate_vector_s{
    CpaCyDrbgSecStrength secStrength;
    CpaBoolean predictionResistanceRequired;

    CpaBoolean isDFUsed;

    Cpa32U entropyInputLen;
    Cpa32U nonceLen;
    Cpa32U persStringLen;

    Cpa8U entropyInput[LAC_DRBG_MAX_SEEDLEN_IN_BYTES];
    Cpa8U nonce[LAC_DRBG_MAX_SEEDLEN_IN_BYTES];
    Cpa8U persString[LAC_DRBG_MAX_SEEDLEN_IN_BYTES];

    Cpa8U v[LAC_DRBG_OUTLEN_IN_BYTES];
    Cpa8U key[LAC_DRBG_KEYLEN256_IN_BYTES];
}lac_drbg_ht_instantiate_vector_t;

/* Vector definition for Generate function health tests */
typedef struct lac_drbg_ht_generate_vector_s{
    CpaBoolean isDFUsed;

    CpaBoolean predictionResistanceEnabled;
    lac_drbg_working_state_t initialState;

    CpaBoolean predictionResistanceRequired;
    CpaCyDrbgSecStrength requestedStrength;

    Cpa32U additionalInputLen;
    Cpa8U additionalInput[LAC_DRBG_MAX_SEEDLEN_IN_BYTES];

    Cpa32U entropyInputLen;
    Cpa8U entropyInput[LAC_DRBG_MAX_SEEDLEN_IN_BYTES];

    Cpa32U requestedBytesLen;
    Cpa8U expectedBytes[LAC_DRBG_OUTLEN_IN_BYTES];

    lac_drbg_working_state_t expectedState;
    CpaStatus expectedStatus;
}lac_drbg_ht_generate_vector_t;

/* Vector definition for Reseed function health tests */
typedef struct lac_drbg_ht_reseed_vector_s{
    CpaBoolean isDFUsed;

    CpaBoolean predictionResistanceEnabled;
    lac_drbg_working_state_t initialState;

    CpaCyDrbgSecStrength setupStrength;

    Cpa32U additionalInputLen;
    Cpa8U additionalInput[LAC_DRBG_MAX_SEEDLEN_IN_BYTES];

    Cpa32U entropyInputLen;
    Cpa8U entropyInput[LAC_DRBG_MAX_SEEDLEN_IN_BYTES];

    lac_drbg_working_state_t expectedState;
    CpaStatus expectedStatus;
}lac_drbg_ht_reseed_vector_t;


/******************************************************************************
 * Test vectors for Instantiate functions
 *****************************************************************************/
lac_drbg_ht_instantiate_vector_t instantiateVectors[] = {
        {
                /* Security Strength */
                CPA_CY_RBG_SEC_STRENGTH_112,
                /* Prediction resistance */
                CPA_FALSE,
                /* is DF used */
                CPA_TRUE,
                /* entropy input length*/
                16,
                /* nonce length */
                8,
                /* pers string length */
                16,
                /* entropy input */
                {0x36, 0x2b, 0xac, 0x91, 0x95, 0xd6, 0x2c, 0x08,
                        0x2b, 0xae, 0xf1, 0x81, 0x14, 0x3a, 0xcd, 0xc0},
                /* nonce */
                {0x1e, 0x6d, 0xe4, 0xa2, 0xb0, 0x6d, 0x13, 0x44},
                /* pers string */
                {0x26, 0x8a, 0xa3, 0xdd, 0x88, 0xa2, 0xe7, 0xe3,
                        0xa3, 0xa9, 0xc1, 0xb6, 0x76, 0x82, 0xf4, 0x30},
                /* v */
                {0xe8, 0x4e, 0x63, 0xc0, 0xc3, 0x63, 0x2e, 0x16,
                        0x0e, 0x3d, 0xf2, 0xcb, 0x4c, 0x20, 0x50, 0x75},
                /* key */
                {0x38, 0xe6, 0xa4, 0x79, 0xff, 0x1c, 0xfa, 0x1e,
                        0xcf, 0x60, 0x7d, 0x1f, 0xef, 0x33, 0xdd, 0xd6}
        },
        {
            /*Security strength*/
            CPA_CY_RBG_SEC_STRENGTH_128,
            /* Prediction resistance */
            CPA_FALSE,
            /* is DF used */
            CPA_TRUE,
            /* entropy input length*/
            16,
            /* nonce length */
            8,
            /* pers string length */
            16,
            /* entropy input */
            {0xad, 0xc9, 0xa4, 0x91, 0x9c, 0xd1, 0x7d, 0x4d,
                    0x71, 0xb6, 0xc6, 0xe0, 0x00, 0x3f, 0x8c, 0xd3},
            /* nonce */
            {0xa4, 0x89, 0x3b, 0x9e, 0x5f, 0x47, 0x12, 0x9d},
            /* pers string */
            {0x2d, 0xe7, 0x62, 0x6f, 0x45, 0xfe, 0x10, 0x7f,
                    0xe7, 0x17, 0xca, 0xa4, 0x40, 0x5e, 0x50, 0x03},
            /* v */
            {0xcf, 0x96, 0xba, 0xcc, 0x99, 0xb6, 0x48, 0xf0,
                    0x8e, 0x93, 0xf0, 0x8f, 0xc7, 0x5e, 0x09, 0x23},
            /* key */
            {0x47, 0xab, 0x7a, 0x37, 0x32, 0xa2, 0x75, 0xa4,
                    0xca, 0x91, 0x67, 0xc4, 0xb2, 0xbb, 0x75, 0xef}
        },
        {
            /*Security strength*/
            CPA_CY_RBG_SEC_STRENGTH_112,
            /*Security strength*/
            /* Prediction resistance */
            CPA_TRUE,
            /* is DF used */
            CPA_TRUE,
            /* entropy input length*/
            16,
            /* nonce length */
            8,
            /* pers string length */
            16,
            /* entropy input */
            {0x55, 0x44, 0x50, 0x4d, 0xc9, 0xa6, 0x1a, 0xba,
                    0x97, 0x4a, 0x4d, 0xb5, 0xe9, 0xb1, 0x5a, 0x90},
            /* nonce */
            {0x34, 0xc5, 0xb9, 0x90, 0x83, 0xff, 0x7f, 0xae},
            /* pers string */
            {0xa0, 0x80, 0x3d, 0xef, 0xa0, 0x26, 0xea, 0x03,
                    0x82, 0x2c, 0x48, 0xc0, 0x4d, 0x2e, 0x63, 0x9c},
            /* v */
            {0xdd, 0x2e, 0x14, 0x28, 0xa9, 0xaa, 0x67, 0x80,
                    0xe9, 0x5f, 0x11, 0x5f, 0x03, 0x8b, 0x3e, 0xa9},
            /* key */
            {0x71, 0x8c, 0xa2, 0xbf, 0x07, 0x9e, 0x20, 0xb8,
                    0x67, 0x95, 0x05, 0xfc, 0x46, 0x42, 0x23, 0x51}
        },
        {
            /*Security strength*/
            CPA_CY_RBG_SEC_STRENGTH_128,
            /* Prediction resistance */
            CPA_TRUE,
            /* is DF used */
            CPA_TRUE,
            /* entropy input length*/
            16,
            /* nonce length */
            8,
            /* pers string length */
            16,
            /* entropy input */
            {0xe7, 0xfb, 0xeb, 0x6f, 0x60, 0x8f, 0xfb, 0xcf,
                    0x54, 0x5e, 0x8b, 0x82, 0xf1, 0xa7, 0x07, 0xd7},
            /* nonce */
            {0x3f, 0x26, 0xfd, 0x75, 0xd8, 0xc3, 0x52, 0xae},
            /* pers string */
            {0xdb, 0x2e, 0xc7, 0xf5, 0x43, 0x89, 0xe0, 0xe2,
                    0x0e, 0x4b, 0x8f, 0x31, 0xa5, 0x40, 0x0e, 0x62},
            /* v */
            {0xe5, 0xd2, 0x22, 0x91, 0xf8, 0x76, 0xd7, 0xfe,
                    0x9d, 0xcd, 0x85, 0xea, 0xfe, 0xcf, 0x12, 0x08},
            /* key */
            {0x57, 0xeb, 0x9c, 0x5c, 0xad, 0x6d, 0x0c, 0x81,
                    0x6e, 0x19, 0xe3, 0x54, 0x9f, 0x65, 0x79, 0x4b}
        },
        {
            /*Security strength*/
            CPA_CY_RBG_SEC_STRENGTH_112,
            /*Security strength*/
            /* Prediction resistance */
            CPA_FALSE,
            /* is DF used */
            CPA_FALSE,
            /* entropy input length*/
            48,
            /* nonce length */
            0,
            /* pers string length */
            48,
            /* entropy input */
            {0x67, 0x4f, 0xc0, 0x27, 0x98, 0x50, 0x8d, 0x4f,
                    0x2c, 0x39, 0xf0, 0x9c, 0x6d, 0x1f, 0xaf, 0x8e,
                    0x1b, 0xbb, 0x35, 0x5d, 0x6b, 0xb0, 0x06, 0x29,
                    0xa5, 0x09, 0x7c, 0xfd, 0xe4, 0x01, 0x89, 0x25,
                    0x58, 0xef, 0x65, 0x7c, 0x92, 0xc4, 0x00, 0x55,
                    0x3c, 0x82, 0x49, 0x28, 0x5a, 0x65, 0x62, 0x28},
            /* nonce */
            {0x00},
            /* pers string */
            {0xb9, 0x06, 0x91, 0xe6, 0x39, 0xdc, 0x74, 0x02,
                    0xf3, 0x11, 0xb2, 0x13, 0x19, 0xc8, 0xd7, 0x6f,
                    0x5e, 0xa9, 0xd5, 0x76, 0x0e, 0x6f, 0x60, 0x53,
                    0x7c, 0x53, 0xc0, 0x3a, 0x26, 0x2d, 0x30, 0xb3,
                    0x44, 0x52, 0x1c, 0x83, 0x98, 0x6d, 0xdd, 0x0c,
                    0x9d, 0xca, 0x29, 0x5a, 0x2c, 0x0c, 0xa5, 0x87},
            /* v */
            {0x6e, 0xdd, 0x7a, 0x35, 0x3d, 0x0f, 0xf7, 0x2d,
                    0x70, 0xea, 0x95, 0xfc, 0x03, 0x6f, 0xf2, 0x21},
            /* key */
            {0x8d, 0x46, 0xdb, 0x3a, 0x66, 0xc9, 0xcf, 0xf4,
                    0x76, 0x4b, 0xf6, 0x7e, 0xb0, 0x1c, 0x0b, 0x6a,
                    0x8b, 0xb5, 0xa0, 0x16, 0x28, 0xbf, 0x0d, 0x14,
                    0xde, 0x14, 0x79, 0x14, 0x78, 0xdf, 0x24, 0x8e}
        },
        {
            /*Security strength*/
            CPA_CY_RBG_SEC_STRENGTH_128,
            /*Security strength*/
            /* Prediction resistance */
            CPA_FALSE,
            /* is DF used */
            CPA_FALSE,
            /* entropy input length*/
            48,
            /* nonce length */
            0,
            /* pers string length */
            48,
            /* entropy input */
            {0x56, 0x3b, 0xfd, 0x26, 0x01, 0x98, 0x4f, 0x59,
                    0xac, 0xf0, 0x0c, 0x67, 0xdf, 0x40, 0x2e, 0x1c,
                    0x41, 0x36, 0xd4, 0x67, 0xfe, 0x2a, 0x24, 0x83,
                    0x6b, 0x6c, 0x36, 0x6f, 0x2f, 0x39, 0x48, 0xa8,
                    0x93, 0xcc, 0xc1, 0x76, 0x3f, 0x44, 0x91, 0x9b,
                    0xd6, 0xfc, 0x5f, 0xa9, 0x8a, 0x5f, 0xed, 0xd4},
            /* nonce */
            {0x00},
            /* pers string */
            {0x50, 0xca, 0x65, 0xca, 0x21, 0x26, 0x25, 0xb0,
                    0xa0, 0x47, 0x83, 0x26, 0xa0, 0xda, 0xb1, 0x51,
                    0xe6, 0xcd, 0x23, 0x55, 0x0b, 0xce, 0x08, 0x70,
                    0xe5, 0x7c, 0x50, 0x5a, 0xa2, 0xe8, 0x5e, 0x0f,
                    0xaf, 0xdc, 0x6f, 0x2e, 0x4c, 0x10, 0xa2, 0xcc,
                    0x4e, 0xde, 0x1c, 0x8b, 0x95, 0x56, 0xbc, 0x14},
            /* v */
            {0x4e, 0x70, 0xad, 0x92, 0x44, 0xf2, 0x19, 0x23,
                    0x49, 0x80, 0xb6, 0xac, 0x6a, 0x0f, 0x64, 0x4e},
            /* key */
            {0x55, 0xfe, 0x12, 0x17, 0xe7, 0xfb, 0x5c, 0x50,
                    0xa5, 0xd4, 0x3b, 0xb0, 0xbb, 0x51, 0xec, 0xc6,
                    0x69, 0x5c, 0xb7, 0x0f, 0xb8, 0x84, 0x47, 0x9d,
                    0x89, 0x5e, 0xa3, 0xe6, 0x37, 0x22, 0x8b, 0xbf}
        },
        {
            /*Security strength*/
            CPA_CY_RBG_SEC_STRENGTH_192,
            /*Security strength*/
            /* Prediction resistance */
            CPA_FALSE,
            /* is DF used */
            CPA_FALSE,
            /* entropy input length*/
            48,
            /* nonce length */
            0,
            /* pers string length */
            48,
            /* entropy input */
            {0x4c, 0xad, 0xec, 0x39, 0x88, 0xae, 0xb4, 0x29,
                    0xe4, 0x4c, 0x6a, 0x2a, 0xc2, 0x90, 0xc6, 0x3e,
                    0x8d, 0x8c, 0x8e, 0x50, 0xe4, 0x5f, 0x58, 0x8b,
                    0x4a, 0xa8, 0x82, 0xcc, 0x79, 0xf0, 0xde, 0xed,
                    0x78, 0x0d, 0xa6, 0x3d, 0xf6, 0xb1, 0x56, 0x97,
                    0xca, 0x6e, 0x6f, 0x13, 0xfe, 0x68, 0x61, 0xa5},
            /* nonce */
            {0x00},
            /* pers string */
            {0x9b, 0xc1, 0x78, 0xd9, 0x19, 0x8f, 0xd9, 0xd6,
                    0xde, 0x92, 0x38, 0x6c, 0xf1, 0x4f, 0x9e, 0x32,
                    0x29, 0x17, 0xf2, 0xff, 0xe1, 0x4c, 0x67, 0x8d,
                    0x9c, 0x5c, 0x20, 0x1f, 0x57, 0x9f, 0x61, 0xcb,
                    0xe5, 0x4b, 0x41, 0xc1, 0x7c, 0x30, 0xbc, 0xa6,
                    0xbd, 0x9a, 0xa1, 0xf0, 0xe5, 0xef, 0x5c, 0xe6},
            /* v */
            {0xef, 0x26, 0xe4, 0x36, 0xbd, 0x27, 0xc0, 0x45,
                    0xa6, 0x56, 0x3b, 0x6d, 0x6e, 0x81, 0x08, 0xcd},
            /* key */
            {0x84, 0x63, 0x1e, 0x1b, 0x56, 0x64, 0x5b, 0x46,
                    0x93, 0xbd, 0xe6, 0xb7, 0xf7, 0x14, 0x2b, 0x87,
                    0x6a, 0x3c, 0x3c, 0x92, 0x48, 0x73, 0x54, 0x68,
                    0xd1, 0xba, 0x67, 0x00, 0x94, 0x9c, 0x22, 0x3e}
        },
        {
            /*Security strength*/
            CPA_CY_RBG_SEC_STRENGTH_256,
            /*Security strength*/
            /* Prediction resistance */
            CPA_FALSE,
            /* is DF used */
            CPA_FALSE,
            /* entropy input length*/
            48,
            /* nonce length */
            0,
            /* pers string length */
            48,
            /* entropy input */
            {0x82, 0x08, 0x84, 0x78, 0x13, 0x76, 0xb5, 0x00,
                    0x2f, 0xe6, 0x29, 0x12, 0x31, 0xf2, 0x8c, 0x96,
                    0xb3, 0xa3, 0x00, 0x72, 0x1c, 0x0f, 0xf0, 0xf7,
                    0xa4, 0x4d, 0xca, 0x38, 0x25, 0xa4, 0xb9, 0xda,
                    0xf5, 0xd9, 0x41, 0xf2, 0x6d, 0x60, 0x5c, 0xf9,
                    0x84, 0x1a, 0x21, 0xda, 0x14, 0x02, 0xcc, 0x1b},
            /* nonce */
            {0x00},
            /* pers string */
            {0x32, 0xeb, 0xd7, 0xcb, 0xa4, 0x18, 0x83, 0x86,
                    0x01, 0xca, 0x39, 0x5f, 0xcb, 0x4f, 0x14, 0xc8,
                    0x49, 0x2e, 0xe3, 0x2a, 0xa3, 0x32, 0x5b, 0xb5,
                    0x5c, 0x27, 0x43, 0x85, 0x1e, 0xc8, 0x89, 0x68,
                    0x81, 0x19, 0x75, 0xde, 0x4e, 0x88, 0x54, 0x8d,
                    0xd9, 0x58, 0x76, 0xe6, 0xec, 0x1d, 0x5f, 0xdb},
            /* v */
            {0x06, 0xa0, 0x37, 0xe6, 0x14, 0x4e, 0x22, 0x00,
                    0x8c, 0xe0, 0xa2, 0xb2, 0x8d, 0x19, 0xa6, 0x4e},
            /* key */
            {0xe3, 0xec, 0xd9, 0x48, 0x70, 0x2b, 0x00, 0x3f,
                    0x87, 0x4f, 0xa4, 0xbc, 0x3e, 0x76, 0xeb, 0xd5,
                    0x34, 0x2a, 0xa3, 0x65, 0xf2, 0x5d, 0xc0, 0x2c,
                    0xff, 0x24, 0x4c, 0x6e, 0x81, 0x9f, 0xad, 0xaa}
        },
        {
            /*Security strength*/
            CPA_CY_RBG_SEC_STRENGTH_112,
            /*Security strength*/
            /* Prediction resistance */
            CPA_TRUE,
            /* is DF used */
            CPA_FALSE,
            /* entropy input length*/
            48,
            /* nonce length */
            0,
            /* pers string length */
            48,
            /* entropy input */
            {0x8a, 0x5b, 0xd1, 0x3b, 0xb5, 0x14, 0x3d, 0xf1,
                    0xf0, 0x3a, 0xa4, 0x7d, 0xdd, 0xa4, 0x19, 0x02,
                    0x4e, 0x6e, 0x97, 0x59, 0xae, 0x4d, 0x35, 0x56,
                    0x28, 0xbc, 0x4c, 0x90, 0x97, 0x69, 0x00, 0x3d,
                    0x99, 0xa0, 0x37, 0xfc, 0x2a, 0x95, 0x29, 0x24,
                    0x0e, 0x8b, 0x00, 0x43, 0x0b, 0x70, 0x59, 0x8a},
            /* nonce */
            {0x00},
            /* pers string */
            {0xd0, 0xa9, 0xcb, 0xdb, 0x31, 0xf9, 0xd2, 0xb8,
                    0xbb, 0x40, 0x63, 0x36, 0x8f, 0x10, 0x2a, 0x3c,
                    0x91, 0xd6, 0x9e, 0xaa, 0x0b, 0x13, 0x0b, 0xdb,
                    0xe5, 0x5e, 0x01, 0x63, 0x46, 0xa9, 0x9d, 0x65,
                    0x87, 0x6b, 0xa6, 0x33, 0x7c, 0x7d, 0x8f, 0x34,
                    0x7b, 0x64, 0x9a, 0x15, 0x01, 0x70, 0x40, 0xe9},
            /* v */
            {0x6c, 0xab, 0x92, 0x05, 0x61, 0x4e, 0x8c, 0x64,
                    0xa4, 0x4d, 0x6f, 0xd8, 0x7f, 0x06, 0x2c, 0xed},
            /* key */
            {0x09, 0xfd, 0x90, 0x1b, 0x43, 0xa8, 0xd9, 0xf0,
                    0xe2, 0x19, 0x73, 0xba, 0x96, 0x7f, 0x40, 0xb5,
                    0x11, 0x1f, 0x49, 0xce, 0xe8, 0x3e, 0x55, 0xe3,
                    0xca, 0xac, 0x88, 0x20, 0x6b, 0x33, 0x00, 0x40}
        },
        {
            /*Security strength*/
            CPA_CY_RBG_SEC_STRENGTH_128,
            /*Security strength*/
            /* Prediction resistance */
            CPA_TRUE,
            /* is DF used */
            CPA_FALSE,
            /* entropy input length*/
            48,
            /* nonce length */
            0,
            /* pers string length */
            48,
            /* entropy input */
            {0xe0, 0x25, 0xae, 0x1e, 0xa1, 0xd4, 0x40, 0x47,
                    0x3e, 0x8c, 0x14, 0x6b, 0x6f, 0x82, 0x44, 0x11,
                    0x88, 0x4f, 0x3f, 0xde, 0x47, 0x25, 0x8d, 0xd2,
                    0x41, 0x4c, 0xe5, 0x33, 0x3b, 0x0c, 0x8d, 0x1a,
                    0xec, 0xd2, 0x68, 0x8e, 0xe1, 0x45, 0x1c, 0x13,
                    0x69, 0x24, 0xce, 0x75, 0xb1, 0x0b, 0x3b, 0xa3},
            /* nonce */
            {0x00},
            /* pers string */
            {0x21, 0xff, 0x14, 0x78, 0xfc, 0xec, 0x9c, 0x3a,
                    0xc8, 0xdd, 0x57, 0xeb, 0x7b, 0x99, 0x90, 0xc0,
                    0xff, 0x88, 0xd5, 0x51, 0x0f, 0xd1, 0x3d, 0x00,
                    0x61, 0x16, 0xe7, 0xf7, 0x8e, 0x82, 0xf4, 0xb4,
                    0xb4, 0x5c, 0x2d, 0x4d, 0xb6, 0x31, 0x3e, 0x8c,
                    0x8b, 0x9b, 0x34, 0x36, 0xc2, 0x0c, 0xe7, 0xc2},
            /* v */
            {0x2a, 0xee, 0x46, 0x09, 0x60, 0xd2, 0x08, 0xeb,
                    0x33, 0x1d, 0x0f, 0xcd, 0x06, 0x01, 0xe9, 0xef},
            /* key */
            {0x92, 0xd5, 0x30, 0x9d, 0x9a, 0x7d, 0xea, 0xc4,
                    0x5f, 0x32, 0xf7, 0x71, 0xd0, 0xd0, 0xa7, 0x5a,
                    0xb9, 0x60, 0xaa, 0xb2, 0x05, 0x94, 0xdb, 0xbc,
                    0x27, 0x14, 0xc7, 0x17, 0x0f, 0x7d, 0xe4, 0xb6}
        },
        {
            /*Security strength*/
            CPA_CY_RBG_SEC_STRENGTH_192,
            /*Security strength*/
            /* Prediction resistance */
            CPA_TRUE,
            /* is DF used */
            CPA_FALSE,
            /* entropy input length*/
            48,
            /* nonce length */
            0,
            /* pers string length */
            48,
            /* entropy input */
            {0xbc, 0xf1, 0x53, 0x3d, 0xc8, 0x6d, 0x19, 0x4b,
                    0xd2, 0xf4, 0x40, 0xbf, 0xc2, 0x33, 0x9c, 0x92,
                    0xab, 0x9e, 0x22, 0xe2, 0x14, 0x4b, 0xe1, 0x57,
                    0x41, 0xa9, 0xa2, 0x86, 0x2d, 0x69, 0xd6, 0x6b,
                    0x45, 0xb6, 0x9d, 0x49, 0x7f, 0x38, 0x5b, 0x45,
                    0x53, 0x3f, 0xfe, 0xb8, 0xe6, 0x8c, 0x88, 0x40},
            /* nonce */
            {0x00},
            /* pers string */
            {0xcd, 0x8c, 0x3c, 0x6e, 0x28, 0x70, 0x7b, 0x09,
                    0xd9, 0x00, 0x73, 0x7f, 0x26, 0xb0, 0xb4, 0xdb,
                    0x12, 0xfa, 0xf2, 0xd3, 0xc2, 0xde, 0xa2, 0x5f,
                    0x19, 0xcb, 0xb6, 0x7d, 0x63, 0x56, 0x17, 0x89,
                    0x0b, 0x93, 0xb0, 0xb6, 0x3b, 0x64, 0x2d, 0x9c,
                    0x70, 0xa5, 0xab, 0xff, 0x0d, 0x53, 0x4c, 0x94},
            /* v */
            {0x3c, 0x45, 0x2e, 0x35, 0x73, 0xfa, 0x5c, 0xad,
                    0xf2, 0x38, 0xa0, 0xc9, 0x9e, 0xd9, 0xf1, 0x5a},
            /* key */
            {0x22, 0x72, 0xe5, 0xa8, 0x27, 0x58, 0x54, 0xfb,
                    0xa2, 0x97, 0x87, 0x31, 0x20, 0x48, 0x5b, 0xc2,
                    0x77, 0xc3, 0x90, 0x0c, 0x9b, 0xf5, 0x28, 0x66,
                    0x5f, 0x2c, 0xd1, 0x28, 0xf4, 0xcc, 0x5c, 0xfa}
        },
        {
            /*Security strength*/
            CPA_CY_RBG_SEC_STRENGTH_256,
            /*Security strength*/
            /* Prediction resistance */
            CPA_TRUE,
            /* is DF used */
            CPA_FALSE,
            /* entropy input length*/
            48,
            /* nonce length */
            0,
            /* pers string length */
            48,
            /* entropy input */
            {0x76, 0x67, 0x9d, 0x43, 0x7e, 0x3f, 0xe6, 0xb5,
                    0xef, 0xea, 0xa6, 0x1a, 0xdf, 0x44, 0xb4, 0x5c,
                    0x0c, 0x01, 0x0c, 0x83, 0x02, 0xea, 0xd8, 0xd6,
                    0xe3, 0x22, 0x33, 0x4d, 0xeb, 0x56, 0x44, 0x01,
                    0x92, 0x03, 0xbe, 0x31, 0x36, 0x7e, 0x2e, 0xf9,
                    0x00, 0xf3, 0x68, 0x54, 0xf2, 0xf8, 0x90, 0x69},
            /* nonce */
            {0x00},
            /* pers string */
            {0xc5, 0x87, 0x2b, 0x0f, 0x1b, 0x9e, 0x95, 0x85,
                    0xc4, 0x44, 0x85, 0xa4, 0x3c, 0x0d, 0x21, 0x00,
                    0xd6, 0x75, 0x9f, 0x1a, 0x6d, 0x36, 0xa9, 0x50,
                    0x59, 0x9d, 0x90, 0x3c, 0x5b, 0x2e, 0x28, 0x48,
                    0x9d, 0xd0, 0xf4, 0x0f, 0xb3, 0x60, 0xad, 0x40,
                    0xbe, 0x4e, 0xf0, 0x89, 0x3f, 0x6a, 0x12, 0xe8},
            /* v */
            {0x7d, 0xb3, 0x49, 0xf4, 0xb2, 0xb8, 0xa9, 0xcd,
                    0x6f, 0x1f, 0x6d, 0x53, 0xb8, 0x94, 0xb7, 0x0f},
            /* key */
            {0xe0, 0xef, 0x3c, 0xb7, 0xa2, 0xe4, 0x45, 0x89,
                    0x82, 0xcd, 0x97, 0x4f, 0x27, 0x82, 0xe6, 0xd7,
                    0x14, 0xd3, 0xd3, 0xa4, 0x22, 0xbc, 0x1a, 0xe8,
                    0xbd, 0xf1, 0x66, 0xa2, 0x0a, 0x8b, 0xf1, 0x51}
        }
};

/******************************************************************************
 *  Test vectors for Generate function for entropy sources
 *  that require derivation function
 *****************************************************************************/
lac_drbg_ht_generate_vector_t generateDFVectors[] = {
        /* Vector #0 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x3a, 0x9a, 0x7f, 0xce, 0xfb, 0xe2, 0xcb, 0x33,
                        0x8e, 0xaa, 0xfe, 0x4e, 0x92, 0x93, 0x26, 0x54},
                /*key*/
                {0xeb, 0x0f, 0x88, 0xbd, 0x13, 0x54, 0xc8, 0x09,
                        0xf7, 0xa5, 0x4b, 0x66, 0xaf, 0x29, 0xc2, 0x00},
                /*reseedCounter*/
                1
            },

            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_112,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

            /*entropyInputLen*/
            0,
            /*entropyInput*/
            {0x00},

            /*requestedBytesLen*/
            16,
            /*expectedBytes*/
            {0x35, 0xaf, 0x60, 0x70, 0xc4, 0x5d, 0x61, 0x69,
                    0xfa, 0xf6, 0xda, 0xcb, 0x8e, 0x24, 0x18, 0x4e},

            /*expectedState*/
            {
                /*v*/
                {0x35, 0x54, 0xf0, 0xaf, 0x83, 0xf2, 0x42, 0x28,
                        0xee, 0xf5, 0x5a, 0x54, 0xe5, 0x6a, 0xf9, 0x70},
                /*key*/
                {0x9e, 0x4b, 0x55, 0xd9, 0x5d, 0x67, 0xe2, 0xc2,
                        0xc5, 0x7d, 0x07, 0x2b, 0x90, 0x20, 0xdf, 0xaf},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        /* Vector #1 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x75, 0x41, 0xc2, 0x52, 0x75, 0xfd, 0xf7, 0xcd,
                        0x98, 0x4a, 0xbf, 0x2d, 0xc1, 0x35, 0x26, 0x48},
                /*key*/
                {0x5d, 0x5e, 0xe5, 0x25, 0xe5, 0x6e, 0x95, 0xce,
                        0xda, 0x8c, 0x01, 0xa9, 0x23, 0xea, 0x6f, 0x45},
                /*reseedCounter*/
                1
            },
            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_128,

            /*additionalInputLen*/
            16,
            /*additionalInput*/
            {0x99, 0xbb, 0xcc, 0xcd, 0x8c, 0x0e, 0xc8, 0xbf,
                    0x00, 0x4a, 0x70, 0x5e, 0x27, 0x17, 0x86, 0x4e},

            /*entropyInputLen*/
            0,
            /*entropyInput*/
            {0x00},

            /*requestedBytesLen*/
            16,
            /*expectedBytes*/
            {0x65, 0x16, 0x0a, 0x11, 0x29, 0x1d, 0xb0, 0xb5,
                    0x91, 0x10, 0xd0, 0x4a, 0x8a, 0x38, 0xd9, 0xe4},

            /*expectedState*/
            {
                /*v*/
                {0x13, 0x02, 0x0f, 0x0f, 0x23, 0xf5, 0xba, 0x2c,
                        0x34, 0x2d, 0xf3, 0x5c, 0x86, 0xbb, 0x15, 0x26},
                /*key*/
                {0xa4, 0x69, 0xec, 0x6f, 0x1d, 0x93, 0xbb, 0xa3,
                        0x56, 0x4e, 0xd0, 0x4d, 0xc1, 0x93, 0xfd, 0xcb},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        /* Vector #2 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x13, 0x02, 0x0f, 0x0f, 0x23, 0xf5, 0xba, 0x2c,
                        0x34, 0x2d, 0xf3, 0x5c, 0x86, 0xbb, 0x15, 0x26},
                /*key*/
                {0xa4, 0x69, 0xec, 0x6f, 0x1d, 0x93, 0xbb, 0xa3,
                        0x56, 0x4e, 0xd0, 0x4d, 0xc1, 0x93, 0xfd, 0xcb},
                /*reseedCounter*/
                2
            },
            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_192,

            /*additionalInputLen*/
            16,
            /*additionalInput*/
            {0x43, 0x8e, 0x54, 0x74, 0x8a, 0xc1, 0xb1, 0x45,
                    0x5f, 0x15, 0xb0, 0x19, 0x92, 0x29, 0x40, 0x05},

            /*entropyInputLen*/
            0,
            /*entropyInput*/
            {0x00},

            /*requestedBytesLen*/
            16,
            /*expectedBytes*/
            {0x00},

            /*expectedState*/
            {
                /*v*/
                {0x13, 0x02, 0x0f, 0x0f, 0x23, 0xf5, 0xba, 0x2c,
                        0x34, 0x2d, 0xf3, 0x5c, 0x86, 0xbb, 0x15, 0x26},
                /*key*/
                {0xa4, 0x69, 0xec, 0x6f, 0x1d, 0x93, 0xbb, 0xa3,
                        0x56, 0x4e, 0xd0, 0x4d, 0xc1, 0x93, 0xfd, 0xcb},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_INVALID_PARAM
        },
        /* Vector #3 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x13, 0x02, 0x0f, 0x0f, 0x23, 0xf5, 0xba, 0x2c,
                        0x34, 0x2d, 0xf3, 0x5c, 0x86, 0xbb, 0x15, 0x26},
                /*key*/
                {0xa4, 0x69, 0xec, 0x6f, 0x1d, 0x93, 0xbb, 0xa3,
                        0x56, 0x4e, 0xd0, 0x4d, 0xc1, 0x93, 0xfd, 0xcb},
                /*reseedCounter*/
                2
            },
            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_256,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

            /*entropyInputLen*/
            0,
            /*entropyInput*/
            {0x00},

            /*requestedBytesLen*/
            16,
            /*expectedBytes*/
            {0x00},

            /*expectedState*/
            {
                /*v*/
                {0x13, 0x02, 0x0f, 0x0f, 0x23, 0xf5, 0xba, 0x2c,
                        0x34, 0x2d, 0xf3, 0x5c, 0x86, 0xbb, 0x15, 0x26},
                /*key*/
                {0xa4, 0x69, 0xec, 0x6f, 0x1d, 0x93, 0xbb, 0xa3,
                        0x56, 0x4e, 0xd0, 0x4d, 0xc1, 0x93, 0xfd, 0xcb},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_INVALID_PARAM
        },
        /* Vector #4 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x13, 0x02, 0x0f, 0x0f, 0x23, 0xf5, 0xba, 0x2c,
                        0x34, 0x2d, 0xf3, 0x5c, 0x86, 0xbb, 0x15, 0x26},
                /*key*/
                {0xa4, 0x69, 0xec, 0x6f, 0x1d, 0x93, 0xbb, 0xa3,
                        0x56, 0x4e, 0xd0, 0x4d, 0xc1, 0x93, 0xfd, 0xcb},
                /*reseedCounter*/
                2
            },
            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_112,

            /*additionalInputLen*/
            16,
            /*additionalInput*/
            {0x99, 0xbb, 0xcc, 0xcd, 0x8c, 0x0e, 0xc8, 0xbf,
                    0x00, 0x4a, 0x70, 0x5e, 0x27, 0x17, 0x86, 0x4e},

            /*entropyInputLen*/
            0,
            /*entropyInput*/
            {0x00},

            /*requestedBytesLen*/
            16,
            /*expectedBytes*/
            {0x00},

            /*expectedState*/
            {
                /*v*/
                {0x13, 0x02, 0x0f, 0x0f, 0x23, 0xf5, 0xba, 0x2c,
                        0x34, 0x2d, 0xf3, 0x5c, 0x86, 0xbb, 0x15, 0x26},
                /*key*/
                {0xa4, 0x69, 0xec, 0x6f, 0x1d, 0x93, 0xbb, 0xa3,
                        0x56, 0x4e, 0xd0, 0x4d, 0xc1, 0x93, 0xfd, 0xcb},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_FAIL
        },
        /* Vector #5 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x13, 0x02, 0x0f, 0x0f, 0x23, 0xf5, 0xba, 0x2c,
                        0x34, 0x2d, 0xf3, 0x5c, 0x86, 0xbb, 0x15, 0x26},
                /*key*/
                {0xa4, 0x69, 0xec, 0x6f, 0x1d, 0x93, 0xbb, 0xa3,
                        0x56, 0x4e, 0xd0, 0x4d, 0xc1, 0x93, 0xfd, 0xcb},
                /*reseedCounter*/
                2
            },
            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_128,

            /*additionalInputLen*/
            16,
            /*additionalInput*/
            {0x43, 0x8e, 0x54, 0x74, 0x8a, 0xc1, 0xb1, 0x45,
                    0x5f, 0x15, 0xb0, 0x19, 0x92, 0x29, 0x40, 0x05},

            /*entropyInputLen*/
            0,
            /*entropyInput*/
            {0x00},

            /*requestedBytesLen*/
            16,
            /*expectedBytes*/
            {0x00},

            /*expectedState*/
            {
                /*v*/
                {0x13, 0x02, 0x0f, 0x0f, 0x23, 0xf5, 0xba, 0x2c,
                        0x34, 0x2d, 0xf3, 0x5c, 0x86, 0xbb, 0x15, 0x26},
                /*key*/
                {0xa4, 0x69, 0xec, 0x6f, 0x1d, 0x93, 0xbb, 0xa3,
                        0x56, 0x4e, 0xd0, 0x4d, 0xc1, 0x93, 0xfd, 0xcb},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_FAIL
        },
        /* Vector #6 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x13, 0x02, 0x0f, 0x0f, 0x23, 0xf5, 0xba, 0x2c,
                        0x34, 0x2d, 0xf3, 0x5c, 0x86, 0xbb, 0x15, 0x26},
                /*key*/
                {0xa4, 0x69, 0xec, 0x6f, 0x1d, 0x93, 0xbb, 0xa3,
                        0x56, 0x4e, 0xd0, 0x4d, 0xc1, 0x93, 0xfd, 0xcb},
                /*reseedCounter*/
                2
            },
            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_192,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

            /*entropyInputLen*/
            0,
            /*entropyInput*/
            {0x00},

            /*requestedBytesLen*/
            16,
            /*expectedBytes*/
            {0x00},

            /*expectedState*/
            {
                /*v*/
                {0x13, 0x02, 0x0f, 0x0f, 0x23, 0xf5, 0xba, 0x2c,
                        0x34, 0x2d, 0xf3, 0x5c, 0x86, 0xbb, 0x15, 0x26},
                /*key*/
                {0xa4, 0x69, 0xec, 0x6f, 0x1d, 0x93, 0xbb, 0xa3,
                        0x56, 0x4e, 0xd0, 0x4d, 0xc1, 0x93, 0xfd, 0xcb},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_INVALID_PARAM
        },
        /* Vector #7 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x13, 0x02, 0x0f, 0x0f, 0x23, 0xf5, 0xba, 0x2c,
                        0x34, 0x2d, 0xf3, 0x5c, 0x86, 0xbb, 0x15, 0x26},
                /*key*/
                {0xa4, 0x69, 0xec, 0x6f, 0x1d, 0x93, 0xbb, 0xa3,
                        0x56, 0x4e, 0xd0, 0x4d, 0xc1, 0x93, 0xfd, 0xcb},
                /*reseedCounter*/
                2
            },
            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_256,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

            /*entropyInputLen*/
            0,
            /*entropyInput*/
            {0x00},

            /*requestedBytesLen*/
            16,
            /*expectedBytes*/
            {0x00},

            /*expectedState*/
            {
                /*v*/
                {0x13, 0x02, 0x0f, 0x0f, 0x23, 0xf5, 0xba, 0x2c,
                        0x34, 0x2d, 0xf3, 0x5c, 0x86, 0xbb, 0x15, 0x26},
                /*key*/
                {0xa4, 0x69, 0xec, 0x6f, 0x1d, 0x93, 0xbb, 0xa3,
                        0x56, 0x4e, 0xd0, 0x4d, 0xc1, 0x93, 0xfd, 0xcb},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_INVALID_PARAM
        },
        /* Vector #8 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0xcf, 0x96, 0xba, 0xcc, 0x99, 0xb6, 0x48, 0xf0,
                        0x8e, 0x93, 0xf0, 0x8f, 0xc7, 0x5e, 0x09, 0x23},
                /*key*/
                {0x47, 0xab, 0x7a, 0x37, 0x32, 0xa2, 0x75, 0xa4,
                        0xca, 0x91, 0x67, 0xc4, 0xb2, 0xbb, 0x75, 0xef},
                /*reseedCounter*/
                1
            },
            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_112,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0xff, 0x3e, 0xe4, 0xf4, 0xb9, 0x60, 0x5e, 0x4e,
                     0xc2, 0x38, 0xc0, 0xfc, 0x34, 0x23, 0x79, 0x14},

            /*expectedState*/
            {
                /*v*/
                {0xce, 0xde, 0x1e, 0x49, 0x98, 0xbc, 0xe6, 0xeb,
                        0xe3, 0x31, 0xd3, 0x21, 0x4d, 0x75, 0x66, 0xa6},
                /*key*/
                {0x31, 0x97, 0x78, 0x80, 0x3d, 0x35, 0xdf, 0x4d,
                        0x08, 0x63, 0x83, 0xce, 0x07, 0x06, 0x29, 0x8c},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        /* Vector #9 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0x90, 0x9e, 0x2e, 0x67, 0x63, 0x9b, 0x21, 0xc5,
                        0x47, 0x9a, 0x72, 0x67, 0x2a, 0xd9, 0x10, 0x87},
                /*key*/
                {0x68, 0xc3, 0xd8, 0xdc, 0xff, 0x61, 0xbe, 0xfc,
                        0xa4, 0x36, 0x1a, 0x58, 0xb8, 0x23, 0x91, 0x1a},
                /*reseedCounter*/
                1
            },
            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_128,

            /*additionalInputLen*/
            16,
            /*additionalInput*/
            {0x99, 0xbb, 0xcc, 0xcd, 0x8c, 0x0e, 0xc8, 0xbf,
                    0x00, 0x4a, 0x70, 0x5e, 0x27, 0x17, 0x86, 0x4e},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0xfb, 0x29, 0x84, 0x6d, 0xc1, 0x53, 0x24, 0x76,
                     0xea, 0xc0, 0x41, 0x72, 0x31, 0x01, 0x58, 0x61},

            /*expectedState*/
            {
                /*v*/
                {0xf7, 0x63, 0x2c, 0xfd, 0x18, 0x87, 0x54, 0xe4,
                        0x49, 0x1e, 0xe0, 0xc4, 0x8e, 0x1c, 0x83, 0xa0},
                /*key*/
                {0xc3, 0x9c, 0x54, 0x26, 0xb2, 0x15, 0x71, 0x4d,
                        0xba, 0xeb, 0x4e, 0x96, 0x4f, 0x53, 0x71, 0xf4},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        /* Vector #10 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0xf7, 0x63, 0x2c, 0xfd, 0x18, 0x87, 0x54, 0xe4,
                        0x49, 0x1e, 0xe0, 0xc4, 0x8e, 0x1c, 0x83, 0xa0},
                /*key*/
                {0xc3, 0x9c, 0x54, 0x26, 0xb2, 0x15, 0x71, 0x4d,
                        0xba, 0xeb, 0x4e, 0x96, 0x4f, 0x53, 0x71, 0xf4},
                /*reseedCounter*/
                2
            },
            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_192,

            /*additionalInputLen*/
            16,
            /*additionalInput*/
            {0x43, 0x8e, 0x54, 0x74, 0x8a, 0xc1, 0xb1, 0x45,
                    0x5f, 0x15, 0xb0, 0x19, 0x92, 0x29, 0x40, 0x05},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x00},

            /*expectedState*/
            {
                /*v*/
                {0xf7, 0x63, 0x2c, 0xfd, 0x18, 0x87, 0x54, 0xe4,
                        0x49, 0x1e, 0xe0, 0xc4, 0x8e, 0x1c, 0x83, 0xa0},
                /*key*/
                {0xc3, 0x9c, 0x54, 0x26, 0xb2, 0x15, 0x71, 0x4d,
                        0xba, 0xeb, 0x4e, 0x96, 0x4f, 0x53, 0x71, 0xf4},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_INVALID_PARAM
        },
        /* Vector #11 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0xf7, 0x63, 0x2c, 0xfd, 0x18, 0x87, 0x54, 0xe4,
                        0x49, 0x1e, 0xe0, 0xc4, 0x8e, 0x1c, 0x83, 0xa0},
                /*key*/
                {0xc3, 0x9c, 0x54, 0x26, 0xb2, 0x15, 0x71, 0x4d,
                        0xba, 0xeb, 0x4e, 0x96, 0x4f, 0x53, 0x71, 0xf4},
                /*reseedCounter*/
                2
            },
            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_256,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x00},

            /*expectedState*/
            {
                /*v*/
                {0xf7, 0x63, 0x2c, 0xfd, 0x18, 0x87, 0x54, 0xe4,
                        0x49, 0x1e, 0xe0, 0xc4, 0x8e, 0x1c, 0x83, 0xa0},
                /*key*/
                {0xc3, 0x9c, 0x54, 0x26, 0xb2, 0x15, 0x71, 0x4d,
                        0xba, 0xeb, 0x4e, 0x96, 0x4f, 0x53, 0x71, 0xf4},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_INVALID_PARAM
        },
        /* Vector #12 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0x65, 0x83, 0x09, 0x68, 0x32, 0x79, 0xb7, 0x50,
                        0x5a, 0xd4, 0x05, 0x9e, 0xd7, 0x61, 0xf9, 0x63},
                /*key*/
                {0xfa, 0x6b, 0x91, 0xc3, 0x47, 0xe7, 0xd1, 0xef,
                        0xe4, 0x29, 0xf2, 0x3b, 0x6a, 0x33, 0x32, 0x2e},
                /*reseedCounter*/
                2
            },

            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_112,

            /*additionalInputLen*/
            16,
            /*additionalInput*/
            {0x99, 0xbb, 0xcc, 0xcd, 0x8c, 0x0e, 0xc8, 0xbf,
                    0x00, 0x4a, 0x70, 0x5e, 0x27, 0x17, 0x86, 0x4e},

            /*entropyInputLen*/
            16,
            /*entropyInput*/
            {0xb4, 0xf5, 0x38, 0x9e, 0x14, 0x72, 0x80, 0x0b,
                    0x72, 0xb6, 0x87, 0x25, 0xb9, 0x43, 0x91, 0xe9},
            /*requestedBytesLen*/
            16,
            /*expectedBytes*/
            {0x09, 0xb5, 0x90, 0xe9, 0x5d, 0xb6, 0xb2, 0xd0,
                    0x07, 0x51, 0xd0, 0x90, 0xe2, 0x5f, 0x0a, 0xac},

            /*expectedState*/
            {
                /*v*/
                {0xb8, 0x41, 0xe3, 0x9d, 0xb3, 0xe7, 0xed, 0xb9,
                        0x68, 0x3c, 0xb9, 0x07, 0x8a, 0xae, 0xad, 0x4a},
                /*key*/
                {0xa8, 0x92, 0x67, 0x4d, 0x81, 0xcb, 0x92, 0x12,
                        0xbf, 0x91, 0x97, 0x9a, 0xf9, 0xeb, 0x36, 0xee},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        /* Vector #13 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0xcd, 0x10, 0x32, 0xfc, 0xb8, 0xc2, 0xef, 0xf3,
                        0x59, 0xab, 0x79, 0x57, 0xbc, 0x89, 0x27, 0xdc},
                /*key*/
                {0x5f, 0xec, 0xcf, 0xa3, 0x42, 0xc8, 0x65, 0x45,
                        0x62, 0xac, 0x81, 0x04, 0x73, 0xff, 0x53, 0x18},
                /*reseedCounter*/
                2
            },
            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_128,

            /*additionalInputLen*/
            16,
            /*additionalInput*/
            {0x43, 0x8e, 0x54, 0x74, 0x8a, 0xc1, 0xb1, 0x45,
                    0x5f, 0x15, 0xb0, 0x19, 0x92, 0x29, 0x40, 0x05},

            /*entropyInputLen*/
            16,
            /*entropyInput*/
            {0x76, 0x08, 0xbe, 0xcc, 0xb2, 0xdf, 0xf6, 0xc0,
                    0x39, 0xb9, 0xf1, 0xdc, 0xa0, 0xdd, 0x8c, 0xdc},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x27, 0x15, 0x7c, 0x62, 0x9a, 0x4a, 0x08, 0x38,
                     0xce, 0xb3, 0x5d, 0xb3, 0xa4, 0xa1, 0xd0, 0xbf},

            /*expectedState*/
            {
                /*v*/
                {0xa4, 0x5a, 0xf9, 0x03, 0x6c, 0x06, 0x8d, 0x2b,
                        0xec, 0xe6, 0xa0, 0x69, 0x6c, 0x89, 0x2f, 0x8c},
                /*key*/
                {0xd9, 0xff, 0x66, 0x8e, 0xe4, 0xfe, 0x16, 0x94,
                        0x80, 0x12, 0x31, 0x37, 0x55, 0xe2, 0x3a, 0xc0},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        /* Vector #14 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0xa4, 0x5a, 0xf9, 0x03, 0x6c, 0x06, 0x8d, 0x2b,
                        0xec, 0xe6, 0xa0, 0x69, 0x6c, 0x89, 0x2f, 0x8c},
                /*key*/
                {0xd9, 0xff, 0x66, 0x8e, 0xe4, 0xfe, 0x16, 0x94,
                        0x80, 0x12, 0x31, 0x37, 0x55, 0xe2, 0x3a, 0xc0},
                /*reseedCounter*/
                2
            },
            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_192,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x00},

            /*expectedState*/
            {
                /*v*/
                {0xa4, 0x5a, 0xf9, 0x03, 0x6c, 0x06, 0x8d, 0x2b,
                        0xec, 0xe6, 0xa0, 0x69, 0x6c, 0x89, 0x2f, 0x8c},
                /*key*/
                {0xd9, 0xff, 0x66, 0x8e, 0xe4, 0xfe, 0x16, 0x94,
                        0x80, 0x12, 0x31, 0x37, 0x55, 0xe2, 0x3a, 0xc0},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_INVALID_PARAM
        },
        /* Vector #15 */
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0xa4, 0x5a, 0xf9, 0x03, 0x6c, 0x06, 0x8d, 0x2b,
                        0xec, 0xe6, 0xa0, 0x69, 0x6c, 0x89, 0x2f, 0x8c},
                /*key*/
                {0xd9, 0xff, 0x66, 0x8e, 0xe4, 0xfe, 0x16, 0x94,
                        0x80, 0x12, 0x31, 0x37, 0x55, 0xe2, 0x3a, 0xc0},
                /*reseedCounter*/
                2
            },
            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_256,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x00},

            /*expectedState*/
            {
                /*v*/
                {0xa4, 0x5a, 0xf9, 0x03, 0x6c, 0x06, 0x8d, 0x2b,
                        0xec, 0xe6, 0xa0, 0x69, 0x6c, 0x89, 0x2f, 0x8c},
                /*key*/
                {0xd9, 0xff, 0x66, 0x8e, 0xe4, 0xfe, 0x16, 0x94,
                        0x80, 0x12, 0x31, 0x37, 0x55, 0xe2, 0x3a, 0xc0},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_INVALID_PARAM
        },
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0x3b, 0xbd, 0x66, 0xcc, 0x30, 0xb2, 0x4c, 0x96,
                        0x4a, 0xa6, 0xbf, 0x6a, 0xb1, 0x11, 0x7a, 0xda},
                /*key*/
                {0xf7, 0x59, 0x8b, 0x4d, 0x84, 0x0a, 0xd4, 0xdd,
                        0x3a, 0x18, 0xb4, 0x1f, 0xbf, 0xce, 0xfa, 0x9d},
                /*reseedCounter*/
                LAC_DRBG_SEED_LIFE + 1
            },

            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_128,

            /*additionalInputLen*/
            16,
            /*additionalInput*/
            {0x43, 0x8e, 0x54, 0x74, 0x8a, 0xc1, 0xb1, 0x45,
                    0x5f, 0x15, 0xb0, 0x19, 0x92, 0x29, 0x40, 0x05},

            /*entropyInputLen*/
            16,
            /*entropyInput*/
            {0x8a, 0xac, 0x67, 0xeb, 0x01, 0x78, 0x45, 0x45,
                    0xc9, 0x65, 0x10, 0xfd, 0xf8, 0x2f, 0x6d, 0x39},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0xac, 0x18, 0x08, 0xf2, 0x0a, 0x42, 0x19, 0xfa,
                     0xa1, 0x1b, 0x4f, 0x95, 0x4c, 0x90, 0x86, 0xc4},

            /*expectedState*/
            {
                /*v*/
                {0x08, 0x77, 0x2d, 0x1d, 0x03, 0x2f, 0xdf, 0x6b,
                        0xa7, 0x15, 0xda, 0xe3, 0x57, 0x63, 0x83, 0xc1},
                /*key*/
                {0xfc, 0xa8, 0xf0, 0xa0, 0xbd, 0x35, 0x42, 0xfb,
                        0xb7, 0x26, 0x3b, 0xf9, 0x3c, 0x57, 0x2c, 0xae},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        }
};
/******************************************************************************
 *  Test vectors for Generate function for entropy sources that
 *  do not require derivation function
 *****************************************************************************/
lac_drbg_ht_generate_vector_t generateNoDFVectors[] = {
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x4a, 0x1b, 0x68, 0xf0, 0x7f, 0x19, 0x38, 0xef,
                        0x00, 0x9c, 0x2e, 0x01, 0x98, 0x90, 0xcd, 0xae},
                /*key*/
                {0xdf, 0x59, 0xbf, 0xcf, 0xbe, 0xb2, 0x6c, 0x8c,
                        0xb2, 0x0f, 0x93, 0xc6, 0xae, 0x08, 0x36, 0x83,
                        0x34, 0x36, 0x21, 0xd0, 0xbd, 0x04, 0xbf, 0x0a,
                        0xe9, 0x2a, 0x3e, 0xc8, 0x0e, 0xd3, 0x1f, 0xe9},
                /*reseedCounter*/
                1
            },

            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_112,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0xce, 0x54, 0x2e, 0x1a, 0x4d, 0xd1, 0x88, 0x23,
                     0x56, 0x46, 0xf1, 0x6f, 0x60, 0xf2, 0xe2, 0x74},

            /*expectedState*/
            {
                /*v*/
                {0xd2, 0xf7, 0x23, 0xbb, 0xe3, 0x27, 0xb6, 0x89,
                        0x22, 0x74, 0x98, 0x7a, 0xd7, 0x12, 0xd0, 0xab},
                /*key*/
                {0x39, 0xce, 0x2b, 0xa6, 0xf1, 0x49, 0xe0, 0xa4,
                        0xd1, 0xe1, 0x6f, 0x53, 0x7e, 0x88, 0x4f, 0xe4,
                        0x02, 0x6a, 0xe7, 0x83, 0x9e, 0x29, 0x60, 0xf5,
                        0x89, 0x4d, 0xa0, 0xb7, 0xbf, 0x02, 0x67, 0x85},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x17, 0xe8, 0x21, 0x93, 0x7a, 0xf1, 0xcf, 0xb2,
                        0xe6, 0x29, 0x42, 0xb4, 0xc2, 0xd8, 0x56, 0x46},
                /*key*/
                {0xeb, 0x34, 0xf1, 0xa7, 0x5c, 0xd7, 0x7a, 0x05,
                        0x60, 0xac, 0x7b, 0x8d, 0xce, 0x96, 0xc6, 0x55,
                        0x41, 0x66, 0x5e, 0x74, 0xef, 0x6c, 0x22, 0x04,
                        0x15, 0xd3, 0x3e, 0xcb, 0x71, 0x0a, 0xca, 0x3b},
                /*reseedCounter*/
                1
            },

            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_128,

            /*additionalInputLen*/
            48,
            /*additionalInput*/
            {0xc5, 0xfa, 0xc8, 0x9e, 0x9e, 0x88, 0xf5, 0xa2,
                    0x56, 0xe9, 0xb5, 0x66, 0x9b, 0x49, 0x4a, 0x62,
                    0x69, 0x64, 0x1a, 0xb1, 0x88, 0xb1, 0x4e, 0xd4,
                    0x47, 0xd1, 0xaa, 0xc5, 0x79, 0x38, 0xae, 0x3e,
                    0xaf, 0x1c, 0xec, 0x49, 0xb6, 0x9e, 0xab, 0x0a,
                    0x52, 0x36, 0xbb, 0xcb, 0x3a, 0x62, 0x99, 0x47},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x18, 0x20, 0x20, 0xa7, 0xd1, 0x33, 0x77, 0xd5,
                     0x2e, 0xc6, 0x36, 0xe7, 0x3f, 0x92, 0xae, 0x47},

            /*expectedState*/
            {
                /*v*/
                {0x07, 0xda, 0xf0, 0x9c, 0xd8, 0xbc, 0xa6, 0x88,
                        0xe7, 0x9f, 0x02, 0x88, 0x3f, 0x2b, 0x4b, 0xff},
                /*key*/
                {0xdd, 0xb5, 0xa0, 0x6b, 0x5e, 0x23, 0x23, 0x4b,
                        0x1b, 0x6d, 0xf6, 0x11, 0x48, 0x7e, 0xfc, 0x27,
                        0x66, 0x9f, 0x69, 0x7e, 0x3a, 0x91, 0x14, 0xa6,
                        0x55, 0x5a, 0xc5, 0x30, 0x70, 0xf7, 0x3b, 0x43},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x07, 0xda, 0xf0, 0x9c, 0xd8, 0xbc, 0xa6, 0x88,
                        0xe7, 0x9f, 0x02, 0x88, 0x3f, 0x2b, 0x4b, 0xff},
                /*key*/
                {0xdd, 0xb5, 0xa0, 0x6b, 0x5e, 0x23, 0x23, 0x4b,
                        0x1b, 0x6d, 0xf6, 0x11, 0x48, 0x7e, 0xfc, 0x27,
                        0x66, 0x9f, 0x69, 0x7e, 0x3a, 0x91, 0x14, 0xa6,
                        0x55, 0x5a, 0xc5, 0x30, 0x70, 0xf7, 0x3b, 0x43},
                /*reseedCounter*/
                2
            },

            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_192,

            /*additionalInputLen*/
            48,
            /*additionalInput*/
            {0xf1, 0x85, 0x9f, 0x54, 0xd3, 0xde, 0x27, 0x93,
                    0xcc, 0xb8, 0x73, 0x97, 0x0f, 0x12, 0xd3, 0xd4,
                    0xf1, 0x8a, 0x0d, 0xee, 0x61, 0x8b, 0xf9, 0x82,
                    0xa4, 0xb9, 0x5b, 0x35, 0x53, 0x9b, 0xff, 0xfd,
                    0x98, 0x88, 0x19, 0xae, 0xa7, 0xd5, 0xdc, 0xc6,
                    0x7a, 0x3f, 0x5b, 0x7b, 0xcf, 0x95, 0x9f, 0x04},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0xf6, 0x3a, 0x75, 0x9d, 0xc6, 0x3b, 0x13, 0x17,
                     0x00, 0x20, 0x17, 0xfc, 0xaf, 0x79, 0x75, 0xc7},

            /*expectedState*/
            {
                /*v*/
                {0x5b, 0x0d, 0x1c, 0x33, 0xe7, 0x7a, 0x28, 0xbd,
                        0x21, 0xd7, 0x75, 0xbc, 0x16, 0x58, 0xeb, 0x64},
                /*key*/
                {0x3b, 0x14, 0x8c, 0xb9, 0x6d, 0x76, 0xc3, 0x24,
                        0x34, 0x2f, 0x75, 0x6e, 0xc3, 0x6a, 0x86, 0xd8,
                        0x1b, 0x22, 0xb1, 0x0e, 0x3e, 0x2a, 0xa9, 0x51,
                        0xb0, 0x0b, 0xdd, 0x6e, 0x66, 0xf5, 0xca, 0x9d},
                /*reseedCounter*/
                3
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x5b, 0x0d, 0x1c, 0x33, 0xe7, 0x7a, 0x28, 0xbd,
                        0x21, 0xd7, 0x75, 0xbc, 0x16, 0x58, 0xeb, 0x64},
                /*key*/
                {0x3b, 0x14, 0x8c, 0xb9, 0x6d, 0x76, 0xc3, 0x24,
                        0x34, 0x2f, 0x75, 0x6e, 0xc3, 0x6a, 0x86, 0xd8,
                        0x1b, 0x22, 0xb1, 0x0e, 0x3e, 0x2a, 0xa9, 0x51,
                        0xb0, 0x0b, 0xdd, 0x6e, 0x66, 0xf5, 0xca, 0x9d},
                /*reseedCounter*/
                3
            },

            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_256,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x65, 0xef, 0x1a, 0x64, 0xeb, 0x40, 0xd1, 0x83,
                     0xbb, 0x4b, 0x85, 0x39, 0xcf, 0xc2, 0x44, 0xae},

            /*expectedState*/
            {
                /*v*/
                {0x1e, 0xfa, 0xdc, 0xde, 0xfe, 0x80, 0x12, 0x9a,
                        0xe3, 0x4b, 0x9b, 0x22, 0x3a, 0x35, 0x3f, 0xe3},
                /*key*/
                {0x3b, 0xe4, 0x01, 0x35, 0x43, 0x7c, 0x0b, 0x83,
                        0x54, 0x4f, 0xff, 0xd6, 0x6a, 0xf8, 0x79, 0x9f,
                        0xf2, 0x6d, 0x07, 0x9e, 0x3b, 0x6d, 0xca, 0xf4,
                        0xe3, 0x00, 0x2a, 0x05, 0x26, 0xd2, 0x8b, 0xe9},
                /*reseedCounter*/
                4
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x1e, 0xfa, 0xdc, 0xde, 0xfe, 0x80, 0x12, 0x9a,
                        0xe3, 0x4b, 0x9b, 0x22, 0x3a, 0x35, 0x3f, 0xe3},
                /*key*/
                {0x3b, 0xe4, 0x01, 0x35, 0x43, 0x7c, 0x0b, 0x83,
                        0x54, 0x4f, 0xff, 0xd6, 0x6a, 0xf8, 0x79, 0x9f,
                        0xf2, 0x6d, 0x07, 0x9e, 0x3b, 0x6d, 0xca, 0xf4,
                        0xe3, 0x00, 0x2a, 0x05, 0x26, 0xd2, 0x8b, 0xe9},
                /*reseedCounter*/
                4
            },

            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_112,

            /*additionalInputLen*/
            48,
            /*additionalInput*/
            {0xc5, 0xfa, 0xc8, 0x9e, 0x9e, 0x88, 0xf5, 0xa2,
                    0x56, 0xe9, 0xb5, 0x66, 0x9b, 0x49, 0x4a, 0x62,
                    0x69, 0x64, 0x1a, 0xb1, 0x88, 0xb1, 0x4e, 0xd4,
                    0x47, 0xd1, 0xaa, 0xc5, 0x79, 0x38, 0xae, 0x3e,
                    0xaf, 0x1c, 0xec, 0x49, 0xb6, 0x9e, 0xab, 0x0a,
                    0x52, 0x36, 0xbb, 0xcb, 0x3a, 0x62, 0x99, 0x47},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x00},

            /*expectedState*/
            {
                /*v*/
                {0x1e, 0xfa, 0xdc, 0xde, 0xfe, 0x80, 0x12, 0x9a,
                        0xe3, 0x4b, 0x9b, 0x22, 0x3a, 0x35, 0x3f, 0xe3},
                /*key*/
                {0x3b, 0xe4, 0x01, 0x35, 0x43, 0x7c, 0x0b, 0x83,
                        0x54, 0x4f, 0xff, 0xd6, 0x6a, 0xf8, 0x79, 0x9f,
                        0xf2, 0x6d, 0x07, 0x9e, 0x3b, 0x6d, 0xca, 0xf4,
                        0xe3, 0x00, 0x2a, 0x05, 0x26, 0xd2, 0x8b, 0xe9},
                /*reseedCounter*/
                4
            },
            /*expectedStatus*/
            CPA_STATUS_FAIL
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x1e, 0xfa, 0xdc, 0xde, 0xfe, 0x80, 0x12, 0x9a,
                        0xe3, 0x4b, 0x9b, 0x22, 0x3a, 0x35, 0x3f, 0xe3},
                /*key*/
                {0x3b, 0xe4, 0x01, 0x35, 0x43, 0x7c, 0x0b, 0x83,
                        0x54, 0x4f, 0xff, 0xd6, 0x6a, 0xf8, 0x79, 0x9f,
                        0xf2, 0x6d, 0x07, 0x9e, 0x3b, 0x6d, 0xca, 0xf4,
                        0xe3, 0x00, 0x2a, 0x05, 0x26, 0xd2, 0x8b, 0xe9},
                /*reseedCounter*/
                4
            },

            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_128,

            /*additionalInputLen*/
            48,
            /*additionalInput*/
            {0xf1, 0x85, 0x9f, 0x54, 0xd3, 0xde, 0x27, 0x93,
                    0xcc, 0xb8, 0x73, 0x97, 0x0f, 0x12, 0xd3, 0xd4,
                    0xf1, 0x8a, 0x0d, 0xee, 0x61, 0x8b, 0xf9, 0x82,
                    0xa4, 0xb9, 0x5b, 0x35, 0x53, 0x9b, 0xff, 0xfd,
                    0x98, 0x88, 0x19, 0xae, 0xa7, 0xd5, 0xdc, 0xc6,
                    0x7a, 0x3f, 0x5b, 0x7b, 0xcf, 0x95, 0x9f, 0x04},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x00},

            /*expectedState*/
            {
                /*v*/
                {0x1e, 0xfa, 0xdc, 0xde, 0xfe, 0x80, 0x12, 0x9a,
                        0xe3, 0x4b, 0x9b, 0x22, 0x3a, 0x35, 0x3f, 0xe3},
                /*key*/
                {0x3b, 0xe4, 0x01, 0x35, 0x43, 0x7c, 0x0b, 0x83,
                        0x54, 0x4f, 0xff, 0xd6, 0x6a, 0xf8, 0x79, 0x9f,
                        0xf2, 0x6d, 0x07, 0x9e, 0x3b, 0x6d, 0xca, 0xf4,
                        0xe3, 0x00, 0x2a, 0x05, 0x26, 0xd2, 0x8b, 0xe9},
                /*reseedCounter*/
                4
            },
            /*expectedStatus*/
            CPA_STATUS_FAIL
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x1e, 0xfa, 0xdc, 0xde, 0xfe, 0x80, 0x12, 0x9a,
                        0xe3, 0x4b, 0x9b, 0x22, 0x3a, 0x35, 0x3f, 0xe3},
                /*key*/
                {0x3b, 0xe4, 0x01, 0x35, 0x43, 0x7c, 0x0b, 0x83,
                        0x54, 0x4f, 0xff, 0xd6, 0x6a, 0xf8, 0x79, 0x9f,
                        0xf2, 0x6d, 0x07, 0x9e, 0x3b, 0x6d, 0xca, 0xf4,
                        0xe3, 0x00, 0x2a, 0x05, 0x26, 0xd2, 0x8b, 0xe9},
                /*reseedCounter*/
                4
            },

            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_192,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x00},

            /*expectedState*/
            {
                /*v*/
                {0x1e, 0xfa, 0xdc, 0xde, 0xfe, 0x80, 0x12, 0x9a,
                        0xe3, 0x4b, 0x9b, 0x22, 0x3a, 0x35, 0x3f, 0xe3},
                /*key*/
                {0x3b, 0xe4, 0x01, 0x35, 0x43, 0x7c, 0x0b, 0x83,
                        0x54, 0x4f, 0xff, 0xd6, 0x6a, 0xf8, 0x79, 0x9f,
                        0xf2, 0x6d, 0x07, 0x9e, 0x3b, 0x6d, 0xca, 0xf4,
                        0xe3, 0x00, 0x2a, 0x05, 0x26, 0xd2, 0x8b, 0xe9},
                /*reseedCounter*/
                4
            },
            /*expectedStatus*/
            CPA_STATUS_FAIL
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                /*v*/
                {0x1e, 0xfa, 0xdc, 0xde, 0xfe, 0x80, 0x12, 0x9a,
                        0xe3, 0x4b, 0x9b, 0x22, 0x3a, 0x35, 0x3f, 0xe3},
                /*key*/
                {0x3b, 0xe4, 0x01, 0x35, 0x43, 0x7c, 0x0b, 0x83,
                        0x54, 0x4f, 0xff, 0xd6, 0x6a, 0xf8, 0x79, 0x9f,
                        0xf2, 0x6d, 0x07, 0x9e, 0x3b, 0x6d, 0xca, 0xf4,
                        0xe3, 0x00, 0x2a, 0x05, 0x26, 0xd2, 0x8b, 0xe9},
                /*reseedCounter*/
                4
            },

            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_256,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x00},

            /*expectedState*/
            {
                /*v*/
                {0x1e, 0xfa, 0xdc, 0xde, 0xfe, 0x80, 0x12, 0x9a,
                        0xe3, 0x4b, 0x9b, 0x22, 0x3a, 0x35, 0x3f, 0xe3},
                /*key*/
                {0x3b, 0xe4, 0x01, 0x35, 0x43, 0x7c, 0x0b, 0x83,
                        0x54, 0x4f, 0xff, 0xd6, 0x6a, 0xf8, 0x79, 0x9f,
                        0xf2, 0x6d, 0x07, 0x9e, 0x3b, 0x6d, 0xca, 0xf4,
                        0xe3, 0x00, 0x2a, 0x05, 0x26, 0xd2, 0x8b, 0xe9},
                /*reseedCounter*/
                4
            },
            /*expectedStatus*/
            CPA_STATUS_FAIL
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0xd6, 0x4b, 0xe2, 0x33, 0x37, 0x82, 0x69, 0xe0,
                        0xd3, 0x68, 0xa1, 0xe1, 0x8d, 0x5d, 0xef, 0x82},
                /*key*/
                {0xf6, 0xb2, 0x0b, 0xf5, 0x28, 0xf2, 0x65, 0xeb,
                        0x8e, 0x9a, 0x23, 0x38, 0x6a, 0xab, 0xfe, 0xf5,
                        0x8d, 0xf3, 0x8a, 0x19, 0xa2, 0x58, 0x4f, 0x04,
                        0x4d, 0x15, 0xe8, 0xd7, 0x74, 0xba, 0x55, 0xae},
                /*reseedCounter*/
                1
            },

            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_112,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0xf4, 0x6e, 0xa6, 0xa1, 0x7a, 0xb4, 0x7f, 0xa1,
                     0x35, 0x6a, 0x04, 0x8b, 0x91, 0xdd, 0x80, 0xbe},

            /*expectedState*/
            {
                /*v*/
                {0xfa, 0xf0, 0x68, 0x3f, 0x1e, 0x24, 0x24, 0x1c,
                        0xcf, 0x89, 0xf7, 0xf4, 0x0c, 0x52, 0x27, 0xb9},
                /*key*/
                {0xd5, 0x22, 0x1b, 0xfc, 0x3b, 0xe0, 0x43, 0x58,
                        0x7d, 0x24, 0xf1, 0x4b, 0x66, 0x84, 0xad, 0xcf,
                        0x8f, 0xde, 0x5f, 0xbd, 0x0f, 0x16, 0x23, 0x06,
                        0x7c, 0xaa, 0x18, 0xdf, 0x24, 0x84, 0x2d, 0x73},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0x55, 0x5b, 0x0a, 0x95, 0x49, 0x99, 0x48, 0x2e,
                        0x0a, 0xa4, 0x60, 0x12, 0xc7, 0x45, 0x7f, 0xe7},
                /*key*/
                {0xad, 0x47, 0xeb, 0xcd, 0xd3, 0x15, 0xa5, 0x62,
                        0x4a, 0x93, 0x71, 0x33, 0xa0, 0xe5, 0x8e, 0x34,
                        0xc7, 0xdc, 0x84, 0x4c, 0x83, 0x93, 0xae, 0x74,
                        0x59, 0x05, 0x6b, 0x76, 0x44, 0x18, 0x7e, 0x74},
                /*reseedCounter*/
                1
            },

            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_128,

            /*additionalInputLen*/
            48,
            /*additionalInput*/
            {0xc5, 0xfa, 0xc8, 0x9e, 0x9e, 0x88, 0xf5, 0xa2,
                    0x56, 0xe9, 0xb5, 0x66, 0x9b, 0x49, 0x4a, 0x62,
                    0x69, 0x64, 0x1a, 0xb1, 0x88, 0xb1, 0x4e, 0xd4,
                    0x47, 0xd1, 0xaa, 0xc5, 0x79, 0x38, 0xae, 0x3e,
                    0xaf, 0x1c, 0xec, 0x49, 0xb6, 0x9e, 0xab, 0x0a,
                    0x52, 0x36, 0xbb, 0xcb, 0x3a, 0x62, 0x99, 0x47},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0xaf, 0x14, 0xa2, 0x5f, 0xbb, 0xf8, 0xdb, 0x67,
                     0x58, 0xa1, 0x7e, 0x12, 0x6c, 0xf1, 0x1d, 0x52},

            /*expectedState*/
            {
                /*v*/
                {0xfc, 0x0f, 0x7d, 0xa5, 0x77, 0x07, 0xb3, 0xe8,
                        0x18, 0xa8, 0x75, 0x6e, 0x2a, 0x44, 0x24, 0x23},
                /*key*/
                {0x54, 0x90, 0x33, 0x49, 0x52, 0xf7, 0xb7, 0x42,
                        0x44, 0xbd, 0xce, 0xee, 0xca, 0x1a, 0x03, 0x97,
                        0x46, 0x7d, 0x22, 0x7e, 0x6b, 0x81, 0xd6, 0x86,
                        0x45, 0x2a, 0x20, 0x7b, 0x0b, 0x05, 0x8a, 0xdd},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0xfc, 0x0f, 0x7d, 0xa5, 0x77, 0x07, 0xb3, 0xe8,
                        0x18, 0xa8, 0x75, 0x6e, 0x2a, 0x44, 0x24, 0x23},
                /*key*/
                {0x54, 0x90, 0x33, 0x49, 0x52, 0xf7, 0xb7, 0x42,
                        0x44, 0xbd, 0xce, 0xee, 0xca, 0x1a, 0x03, 0x97,
                        0x46, 0x7d, 0x22, 0x7e, 0x6b, 0x81, 0xd6, 0x86,
                        0x45, 0x2a, 0x20, 0x7b, 0x0b, 0x05, 0x8a, 0xdd},
                /*reseedCounter*/
                2
            },

            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_192,

            /*additionalInputLen*/
            48,
            /*additionalInput*/
            {0xf1, 0x85, 0x9f, 0x54, 0xd3, 0xde, 0x27, 0x93,
                    0xcc, 0xb8, 0x73, 0x97, 0x0f, 0x12, 0xd3, 0xd4,
                    0xf1, 0x8a, 0x0d, 0xee, 0x61, 0x8b, 0xf9, 0x82,
                    0xa4, 0xb9, 0x5b, 0x35, 0x53, 0x9b, 0xff, 0xfd,
                    0x98, 0x88, 0x19, 0xae, 0xa7, 0xd5, 0xdc, 0xc6,
                    0x7a, 0x3f, 0x5b, 0x7b, 0xcf, 0x95, 0x9f, 0x04},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0xd9, 0xfa, 0xa5, 0x2f, 0xca, 0xfc, 0xc0, 0x17,
                     0xa2, 0x27, 0x76, 0xf7, 0xcc, 0xa9, 0x76, 0x41},

            /*expectedState*/
            {
                /*v*/
                {0x93, 0x09, 0x4f, 0x87, 0x79, 0x23, 0x24, 0x47,
                        0x77, 0x68, 0x5c, 0x92, 0xad, 0x77, 0x7c, 0xc4},
                /*key*/
                {0x1e, 0x66, 0x05, 0x79, 0x1e, 0x72, 0x9e, 0xab,
                        0xff, 0x08, 0xd3, 0x27, 0x29, 0x93, 0x1d, 0xf1,
                        0xb5, 0x7a, 0x5a, 0x04, 0x70, 0x13, 0x83, 0x0b,
                        0x82, 0x1a, 0x68, 0xf0, 0x78, 0x0f, 0xed, 0x59},
                /*reseedCounter*/
                3
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0x93, 0x09, 0x4f, 0x87, 0x79, 0x23, 0x24, 0x47,
                        0x77, 0x68, 0x5c, 0x92, 0xad, 0x77, 0x7c, 0xc4},
                /*key*/
                {0x1e, 0x66, 0x05, 0x79, 0x1e, 0x72, 0x9e, 0xab,
                        0xff, 0x08, 0xd3, 0x27, 0x29, 0x93, 0x1d, 0xf1,
                        0xb5, 0x7a, 0x5a, 0x04, 0x70, 0x13, 0x83, 0x0b,
                        0x82, 0x1a, 0x68, 0xf0, 0x78, 0x0f, 0xed, 0x59},
                /*reseedCounter*/
                3
            },

            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_256,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

             /*entropyInputLen*/
             0,
             /*entropyInput*/
             {0x00},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x8f, 0x86, 0x23, 0xfb, 0x19, 0x93, 0x46, 0xac,
                     0x0e, 0x07, 0x1f, 0xa1, 0x9d, 0xed, 0xe6, 0x79},

            /*expectedState*/
            {
                /*v*/
                {0x79, 0xf0, 0x85, 0x1b, 0x1a, 0x91, 0xd1, 0x8c,
                        0x04, 0xef, 0x0a, 0x0f, 0x1f, 0x46, 0xb3, 0x8e},
                /*key*/
                {0x76, 0x23, 0x80, 0x57, 0xff, 0xb8, 0x9f, 0x85,
                        0x70, 0x36, 0x46, 0x5c, 0x30, 0xc1, 0xe5, 0xb5,
                        0xd9, 0x6b, 0xd5, 0x3d, 0x2c, 0x7b, 0x9a, 0x44,
                        0x67, 0xd3, 0xca, 0x9e, 0xd7, 0x2e, 0x33, 0x57},
                /*reseedCounter*/
                4
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0x79, 0xf0, 0x85, 0x1b, 0x1a, 0x91, 0xd1, 0x8c,
                        0x04, 0xef, 0x0a, 0x0f, 0x1f, 0x46, 0xb3, 0x8e},
                /*key*/
                {0x76, 0x23, 0x80, 0x57, 0xff, 0xb8, 0x9f, 0x85,
                        0x70, 0x36, 0x46, 0x5c, 0x30, 0xc1, 0xe5, 0xb5,
                        0xd9, 0x6b, 0xd5, 0x3d, 0x2c, 0x7b, 0x9a, 0x44,
                        0x67, 0xd3, 0xca, 0x9e, 0xd7, 0x2e, 0x33, 0x57},
                /*reseedCounter*/
                4
            },

            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_112,

            /*additionalInputLen*/
            48,
            /*additionalInput*/
            {0xc5, 0xfa, 0xc8, 0x9e, 0x9e, 0x88, 0xf5, 0xa2,
                    0x56, 0xe9, 0xb5, 0x66, 0x9b, 0x49, 0x4a, 0x62,
                    0x69, 0x64, 0x1a, 0xb1, 0x88, 0xb1, 0x4e, 0xd4,
                    0x47, 0xd1, 0xaa, 0xc5, 0x79, 0x38, 0xae, 0x3e,
                    0xaf, 0x1c, 0xec, 0x49, 0xb6, 0x9e, 0xab, 0x0a,
                    0x52, 0x36, 0xbb, 0xcb, 0x3a, 0x62, 0x99, 0x47},

            /*entropyInputLen*/
            48,
            /*entropyInput*/
            {0x61, 0xd9, 0x1e, 0xc0, 0x6e, 0x24, 0xc8, 0x7b,
                    0x30, 0xeb, 0xc1, 0x5a, 0xa4, 0xaa, 0x23, 0x4e,
                    0x09, 0x4f, 0x4d, 0x10, 0x68, 0xa3, 0xc3, 0xe2,
                    0xcc, 0x90, 0xb6, 0xdc, 0x19, 0xd7, 0xce, 0x67,
                    0x00, 0x6c, 0x22, 0x64, 0xbb, 0x2a, 0x97, 0x24,
                    0xf4, 0x5f, 0x6c, 0x4f, 0x1c, 0x0e, 0x85, 0x02},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x95, 0x04, 0x47, 0x93, 0x48, 0x6d, 0xe7, 0xe6,
                     0xca, 0x6a, 0xa7, 0x18, 0xbc, 0x3a, 0x65, 0x80},

            /*expectedState*/
            {
                /*v*/
                {0x4e, 0xf8, 0xef, 0xa5, 0x5d, 0x4e, 0x14, 0xad,
                        0x61, 0xf4, 0xfa, 0x4a, 0x83, 0xf8, 0xe9, 0x0d},
                /*key*/
                {0x11, 0x06, 0x1c, 0x0e, 0xc4, 0xa2, 0xe6, 0x9a,
                        0x18, 0x00, 0x94, 0xb9, 0x2f, 0x13, 0x5c, 0x37,
                        0x45, 0xf6, 0x77, 0x69, 0x7b, 0x92, 0xb4, 0xba,
                        0x6a, 0xcc, 0x5e, 0x54, 0xc5, 0x82, 0x6b, 0x39},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0x4e, 0xf8, 0xef, 0xa5, 0x5d, 0x4e, 0x14, 0xad,
                        0x61, 0xf4, 0xfa, 0x4a, 0x83, 0xf8, 0xe9, 0x0d},
                /*key*/
                {0x11, 0x06, 0x1c, 0x0e, 0xc4, 0xa2, 0xe6, 0x9a,
                        0x18, 0x00, 0x94, 0xb9, 0x2f, 0x13, 0x5c, 0x37,
                        0x45, 0xf6, 0x77, 0x69, 0x7b, 0x92, 0xb4, 0xba,
                        0x6a, 0xcc, 0x5e, 0x54, 0xc5, 0x82, 0x6b, 0x39},
                /*reseedCounter*/
                2
            },

            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_128,

            /*additionalInputLen*/
            48,
            /*additionalInput*/
            {0xf1, 0x85, 0x9f, 0x54, 0xd3, 0xde, 0x27, 0x93,
                    0xcc, 0xb8, 0x73, 0x97, 0x0f, 0x12, 0xd3, 0xd4,
                    0xf1, 0x8a, 0x0d, 0xee, 0x61, 0x8b, 0xf9, 0x82,
                    0xa4, 0xb9, 0x5b, 0x35, 0x53, 0x9b, 0xff, 0xfd,
                    0x98, 0x88, 0x19, 0xae, 0xa7, 0xd5, 0xdc, 0xc6,
                    0x7a, 0x3f, 0x5b, 0x7b, 0xcf, 0x95, 0x9f, 0x04},

            /*entropyInputLen*/
            48,
            /*entropyInput*/
            {0x4e, 0x97, 0xa0, 0xcc, 0x78, 0xa1, 0xb8, 0xd9,
                    0x2e, 0xd4, 0x9b, 0x06, 0xa1, 0x43, 0x39, 0xbc,
                    0x72, 0xbb, 0x1d, 0xad, 0x06, 0x7f, 0xf0, 0xfc,
                    0x42, 0x95, 0x1b, 0xae, 0x4a, 0xae, 0xc1, 0x9b,
                    0x79, 0xa8, 0xf9, 0xc6, 0x26, 0xcf, 0x43, 0x89,
                    0x82, 0x4d, 0x49, 0xe6, 0x9a, 0x1b, 0x88, 0x53},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x3f, 0xfe, 0x67, 0x74, 0xee, 0x5c, 0xeb, 0x81,
                     0xf8, 0xfb, 0xad, 0xa9, 0x7e, 0x4f, 0xeb, 0xea},

            /*expectedState*/
            {
                /*v*/
                {0x5c, 0x8b, 0xef, 0x4e, 0x9d, 0x3f, 0xbe, 0x30,
                        0x8f, 0xd1, 0x85, 0x0d, 0x1a, 0xb8, 0xa3, 0xdb},
                /*key*/
                {0x71, 0x71, 0x24, 0x2c, 0x29, 0x10, 0xd0, 0xfc,
                        0x5c, 0xa0, 0xeb, 0xff, 0x26, 0xda, 0xa9, 0xb4,
                        0x28, 0x5f, 0xd5, 0x9f, 0x50, 0xce, 0xab, 0x8b,
                        0x72, 0xde, 0x95, 0xb6, 0xcf, 0xe3, 0x0a, 0xa4},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0x5c, 0x8b, 0xef, 0x4e, 0x9d, 0x3f, 0xbe, 0x30,
                        0x8f, 0xd1, 0x85, 0x0d, 0x1a, 0xb8, 0xa3, 0xdb},
                /*key*/
                {0x71, 0x71, 0x24, 0x2c, 0x29, 0x10, 0xd0, 0xfc,
                        0x5c, 0xa0, 0xeb, 0xff, 0x26, 0xda, 0xa9, 0xb4,
                        0x28, 0x5f, 0xd5, 0x9f, 0x50, 0xce, 0xab, 0x8b,
                        0x72, 0xde, 0x95, 0xb6, 0xcf, 0xe3, 0x0a, 0xa4},
                /*reseedCounter*/
                2
            },

            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_192,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

            /*entropyInputLen*/
            48,
            /*entropyInput*/
            {0x9b, 0xd2, 0x96, 0xee, 0xf7, 0x39, 0xad, 0xcf,
                    0x86, 0x17, 0xb0, 0x0d, 0x13, 0x14, 0x6b, 0xed,
                    0x7d, 0xca, 0x62, 0x8f, 0xb8, 0xbb, 0x9d, 0x20,
                    0xe5, 0x00, 0x49, 0x0d, 0xcc, 0xc0, 0xc4, 0x2b,
                    0xc4, 0x77, 0x2b, 0xcb, 0xea, 0x65, 0x2a, 0x3a,
                    0xe7, 0x5f, 0xdb, 0xa9, 0xfa, 0xa6, 0x52, 0xa5},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x63, 0xfb, 0x52, 0x8d, 0x9e, 0xac, 0x95, 0xe2,
                     0x49, 0xa1, 0x98, 0x23, 0x0c, 0x9f, 0x4c, 0x5c},

            /*expectedState*/
            {
                /*v*/
                {0xa5, 0x7b, 0xeb, 0xb8, 0x4a, 0xb3, 0x21, 0xc3,
                        0x84, 0x84, 0xd2, 0x0a, 0x5d, 0x76, 0x63, 0x07},
                /*key*/
                {0x55, 0x4c, 0x26, 0x29, 0x95, 0x1b, 0xae, 0x82,
                        0x24, 0x26, 0x9c, 0x80, 0xd4, 0x19, 0xd8, 0xfe,
                        0x1c, 0xec, 0xc3, 0xbc, 0x02, 0x1a, 0x37, 0x41,
                        0x26, 0x1d, 0x36, 0xb5, 0x4f, 0x9f, 0xac, 0x7a},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0xa5, 0x7b, 0xeb, 0xb8, 0x4a, 0xb3, 0x21, 0xc3,
                        0x84, 0x84, 0xd2, 0x0a, 0x5d, 0x76, 0x63, 0x07},
                /*key*/
                {0x55, 0x4c, 0x26, 0x29, 0x95, 0x1b, 0xae, 0x82,
                        0x24, 0x26, 0x9c, 0x80, 0xd4, 0x19, 0xd8, 0xfe,
                        0x1c, 0xec, 0xc3, 0xbc, 0x02, 0x1a, 0x37, 0x41,
                        0x26, 0x1d, 0x36, 0xb5, 0x4f, 0x9f, 0xac, 0x7a},
                /*reseedCounter*/
                2
            },

            /*predictionResistanceRequired*/
            CPA_TRUE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_256,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {0x00},

            /*entropyInputLen*/
            48,
            /*entropyInput*/
            {0x51, 0x3c, 0x7b, 0x8f, 0x08, 0xb3, 0x8f, 0xef,
                    0x71, 0xa0, 0x86, 0x35, 0x58, 0x23, 0xbe, 0x24,
                    0x2a, 0xa4, 0xbc, 0x04, 0x2b, 0x3d, 0xe5, 0x93,
                    0x78, 0x70, 0x0d, 0xe1, 0x3e, 0x8f, 0x38, 0xb2,
                    0x84, 0x9b, 0x4a, 0xf3, 0x13, 0x46, 0xf5, 0x93,
                    0x16, 0xc5, 0xb0, 0x8a, 0x0e, 0x5e, 0x07, 0x78},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0xe7, 0x45, 0x1d, 0xef, 0xec, 0x17, 0x27, 0x11,
                     0x3d, 0xcb, 0x6a, 0x6f, 0xed, 0xd8, 0x4c, 0x41},

            /*expectedState*/
            {
                /*v*/
                {0xe5, 0x43, 0x41, 0x41, 0xa5, 0x52, 0xa5, 0xd9,
                        0xa1, 0x3f, 0xbe, 0x09, 0xf0, 0xed, 0x98, 0xf7},
                /*key*/
                {0x94, 0xfc, 0x6c, 0x08, 0x19, 0x7c, 0xf7, 0xe4,
                        0xd1, 0x16, 0x15, 0x25, 0x6b, 0x13, 0xcc, 0x48,
                        0x93, 0xe1, 0x17, 0x7d, 0x77, 0x73, 0xd6, 0x39,
                        0x6e, 0xd3, 0xfa, 0xb5, 0xeb, 0xa3, 0xcc, 0x9f},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                /*v*/
                {0x00, 0xb9, 0xd2, 0xdd, 0x46, 0x95, 0xfa, 0x63,
                        0x45, 0x7b, 0x8e, 0x30, 0x56, 0xbd, 0xab, 0x3b},
                /*key*/
                {0xe0, 0x92, 0x2d, 0xab, 0x52, 0xae, 0x44, 0x02,
                        0x0b, 0x4d, 0x08, 0xfe, 0x5d, 0x6e, 0x25, 0x99,
                        0xa1, 0x91, 0x08, 0xb4, 0x4b, 0x57, 0xee, 0xa7,
                        0xce, 0xaa, 0x03, 0xc9, 0x05, 0x50, 0x9d, 0x25},
                /*reseedCounter*/
                LAC_DRBG_SEED_LIFE + 1
            },

            /*predictionResistanceRequired*/
            CPA_FALSE,
            /*requestedStrength*/
            CPA_CY_RBG_SEC_STRENGTH_256,

            /*additionalInputLen*/
            16,
            /*additionalInput*/
            {0x43, 0x8e, 0x54, 0x74, 0x8a, 0xc1, 0xb1, 0x45,
                    0x5f, 0x15, 0xb0, 0x19, 0x92, 0x29, 0x40, 0x05},

            /*entropyInputLen*/
            48,
            /*entropyInput*/
            {0xf7, 0x23, 0x21, 0xe8, 0x34, 0x2f, 0x97, 0xab,
                    0xd0, 0x6e, 0x13, 0x31, 0x7a, 0xfd, 0x22, 0xd2,
                    0x32, 0xfe, 0x3f, 0xf8, 0xe3, 0x43, 0x45, 0x33,
                    0xce, 0x0b, 0xa6, 0xe8, 0x42, 0xce, 0xe4, 0x60,
                    0xdb, 0x9a, 0xe3, 0x7c, 0x6d, 0x04, 0x87, 0xec,
                    0x91, 0x09, 0xd4, 0x3a, 0xf0, 0x20, 0x07, 0x50},
             /*requestedBytesLen*/
             16,
             /*expectedBytes*/
             {0x52, 0x7f, 0x9e, 0x1f, 0x7d, 0x50, 0x33, 0x6c,
                     0x78, 0x29, 0xb4, 0x8f, 0xe6, 0xff, 0xd1, 0x05},

            /*expectedState*/
            {
                /*v*/
                {0x8a, 0x69, 0xab, 0x80, 0xb8, 0xec, 0xc3, 0x64,
                        0x03, 0x86, 0x7f, 0xb9, 0xd8, 0x03, 0xc0, 0x09},
                /*key*/
                {0x51, 0xbd, 0x4c, 0x97, 0x05, 0xef, 0x1a, 0x42,
                        0x0b, 0x48, 0x7a, 0xeb, 0xdb, 0x41, 0x30, 0xfb,
                        0x6f, 0x96, 0x8a, 0x61, 0x12, 0x84, 0x08, 0xa4,
                        0xc7, 0x11, 0x3b, 0x0a, 0xb3, 0x7f, 0x00, 0x9f},
                /*reseedCounter*/
                2
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        }
};

/******************************************************************************
 *  Test vectors for Reseed function for entropy sources
 *  that require derivation function
 *****************************************************************************/
lac_drbg_ht_reseed_vector_t reseedDFVectors[] = {
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                    /*v*/
                    {0xcf, 0x96, 0xba, 0xcc, 0x99, 0xb6, 0x48, 0xf0,
                            0x8e, 0x93, 0xf0, 0x8f, 0xc7, 0x5e, 0x09, 0x23
                    },
                    /*key*/
                    {0x47, 0xab, 0x7a, 0x37, 0x32, 0xa2, 0x75, 0xa4,
                            0xca, 0x91, 0x67, 0xc4, 0xb2, 0xbb, 0x75, 0xef
                    },
                    /*reseedCounter*/
                    1
            },

            /*setupStrength*/
            CPA_CY_RBG_SEC_STRENGTH_112,

            /*additionalInputLen*/
            16,
            /*additionalInput*/
            {0x2b, 0x00, 0xc1, 0x26, 0x8c, 0x15, 0xce, 0x27,
                    0x93, 0x0f, 0xa9, 0xdf, 0x9d, 0xb4, 0xea, 0x81
            },

            /*entropyInputLen*/
            16,
            /*entropyInput*/
            {0x40, 0xa4, 0x5d, 0xab, 0x46, 0x8b, 0x78, 0x71,
                    0x71, 0x12, 0x08, 0xb9, 0x61, 0x85, 0xc5, 0x16
            },

            /*expectedState*/
            {
                    /*v*/
                    {0xc0, 0x17, 0x67, 0xd4, 0xe9, 0x41, 0x4a, 0xcd,
                            0x18, 0x72, 0x33, 0x7e, 0xd1, 0x71, 0xe3, 0x2a
                    },
                    /*key*/
                    {0xbd, 0x4d, 0x7c, 0x36, 0x27, 0x48, 0x4f, 0x1b,
                            0xc6, 0x3e, 0x0a, 0xc0, 0x1e, 0x72, 0xf6, 0xc3
                    },
                    /*reseedCounter*/
                    1
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                    /*v*/
                    {0xc0, 0x17, 0x67, 0xd4, 0xe9, 0x41, 0x4a, 0xcd,
                            0x18, 0x72, 0x33, 0x7e, 0xd1, 0x71, 0xe3, 0x2a
                    },
                    /*key*/
                    {0xbd, 0x4d, 0x7c, 0x36, 0x27, 0x48, 0x4f, 0x1b,
                            0xc6, 0x3e, 0x0a, 0xc0, 0x1e, 0x72, 0xf6, 0xc3
                    },
                    /*reseedCounter*/
                    1
            },

            /*setupStrength*/
            CPA_CY_RBG_SEC_STRENGTH_112,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {},

            /*entropyInputLen*/
            16,
            /*entropyInput*/
            {0x5c, 0x9f, 0x67, 0x36, 0x30, 0x91, 0xce, 0xcd,
                    0x17, 0xe6, 0x6d, 0x79, 0xc9, 0x09, 0xd7, 0x41
            },

            /*expectedState*/
            {
                    /*v*/
                    {0xc1, 0xf3, 0xc4, 0xc4, 0xcf, 0x27, 0xfd, 0x31,
                            0x50, 0x69, 0x4c, 0xb4, 0x8f, 0x85, 0xf5, 0x96
                    },
                    /*key*/
                    {0xe1, 0x8d, 0x25, 0xbf, 0x38, 0x84, 0x66, 0x89,
                            0xab, 0xcf, 0xb6, 0x02, 0x09, 0x1d, 0x7b, 0xd5
                    },
                    /*reseedCounter*/
                    1
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                    /*v*/
                    {0xcf, 0x96, 0xba, 0xcc, 0x99, 0xb6, 0x48, 0xf0,
                            0x8e, 0x93, 0xf0, 0x8f, 0xc7, 0x5e, 0x09, 0x23
                    },
                    /*key*/
                    {0x47, 0xab, 0x7a, 0x37, 0x32, 0xa2, 0x75, 0xa4,
                            0xca, 0x91, 0x67, 0xc4, 0xb2, 0xbb, 0x75, 0xef
                    },
                    /*reseedCounter*/
                    1
            },

            /*setupStrength*/
            CPA_CY_RBG_SEC_STRENGTH_128,

            /*additionalInputLen*/
            16,
            /*additionalInput*/
            {0xf9, 0x42, 0x46, 0x36, 0x35, 0x26, 0x88, 0x8b,
                    0x43, 0x83, 0x67, 0xce, 0xab, 0xcb, 0x2d, 0x0d
            },

            /*entropyInputLen*/
            16,
            /*entropyInput*/
            {0xd1, 0xcd, 0x71, 0x78, 0x3e, 0x19, 0x7e, 0x68,
                    0xdf, 0x96, 0x20, 0x7a, 0xc6, 0x80, 0x71, 0xeb
            },

            /*expectedState*/
            {
                    /*v*/
                    {0xa4, 0x4b, 0xe3, 0x3c, 0x36, 0xf4, 0xb0, 0xcf,
                            0xd9, 0xfd, 0x0f, 0x44, 0xfd, 0x15, 0xbc, 0x5e
                    },
                    /*key*/
                    {0x62, 0xbb, 0xd6, 0xe5, 0x7a, 0x8f, 0x7f, 0x05,
                            0x1f, 0xb7, 0xcb, 0xc1, 0xe5, 0x6e, 0xc9, 0x8e
                    },
                    /*reseedCounter*/
                    1
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_TRUE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                    /*v*/
                    {0xa4, 0x4b, 0xe3, 0x3c, 0x36, 0xf4, 0xb0, 0xcf,
                            0xd9, 0xfd, 0x0f, 0x44, 0xfd, 0x15, 0xbc, 0x5e
                    },
                    /*key*/
                    {0x62, 0xbb, 0xd6, 0xe5, 0x7a, 0x8f, 0x7f, 0x05,
                            0x1f, 0xb7, 0xcb, 0xc1, 0xe5, 0x6e, 0xc9, 0x8e
                    },
                    /*reseedCounter*/
                    1
            },

            /*setupStrength*/
            CPA_CY_RBG_SEC_STRENGTH_128,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {},

            /*entropyInputLen*/
            16,
            /*entropyInput*/
            {0xf1, 0xc7, 0x47, 0x78, 0xd6, 0x99, 0x78, 0x96,
                    0xc1, 0x51, 0x2f, 0xbd, 0xee, 0xea, 0xfc, 0x1c
            },

            /*expectedState*/
            {
                    /*v*/
                    {0x6b, 0x73, 0x15, 0x7f, 0x16, 0xd3, 0x79, 0x24,
                            0x41, 0xb4, 0x52, 0x94, 0xb5, 0xd7, 0x62, 0x98
                    },
                    /*key*/
                    {0xda, 0xcf, 0xf2, 0x22, 0xb9, 0xda, 0x8e, 0x41,
                            0x80, 0x2e, 0x29, 0x6e, 0x0d, 0x31, 0xca, 0x42
                    },
                    /*reseedCounter*/
                    1
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        }
};

/******************************************************************************
 *  Test vectors for Reseed function for entropy sources
 *  that do not require derivation function
 *****************************************************************************/
lac_drbg_ht_reseed_vector_t reseedNoDFVectors[] = {
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                    /*v*/
                    {0xae, 0xe7, 0x32, 0xc5, 0x93, 0x9e, 0x6f, 0xa7,
                            0xf1, 0x14, 0x6c, 0x9f, 0x60, 0xb7, 0x23, 0x22
                    },
                    /*key*/
                    {0x85, 0x95, 0x68, 0x09, 0xc0, 0xa8, 0x79, 0x41,
                            0x96, 0xd7, 0x86, 0xf1, 0x5a, 0xc4, 0x51, 0x1d,
                            0xe1, 0xa1, 0x30, 0xf8, 0x5e, 0xbc, 0xe0, 0x76,
                            0xfe, 0x35, 0xe4, 0x03, 0x95, 0xf8, 0xe8, 0x69
                    },
                    /*reseedCounter*/
                    1
            },

            /*setupStrength*/
            CPA_CY_RBG_SEC_STRENGTH_112,

            /*additionalInputLen*/
            48,
            /*additionalInput*/
            {0xaf, 0xca, 0x63, 0x32, 0x3f, 0x90, 0xa9, 0xd8,
                    0x7a, 0x63, 0x5a, 0x07, 0x39, 0xa7, 0xaf, 0xd4,
                    0x90, 0xd5, 0xc3, 0x89, 0x87, 0x7c, 0x09, 0xc7,
                    0x1b, 0xd7, 0x40, 0x0c, 0x15, 0xbb, 0x27, 0x31,
                    0x09, 0x2f, 0x44, 0x97, 0x85, 0x66, 0xdf, 0xb7,
                    0xf7, 0xc2, 0xae, 0x16, 0x20, 0x7a, 0xb4, 0xb2
            },

            /*entropyInputLen*/
            48,
            /*entropyInput*/
            {0xb6, 0x5b, 0x86, 0x37, 0x7d, 0xef, 0x95, 0x4a,
                    0xb9, 0x1c, 0x99, 0x80, 0x0b, 0x21, 0xd9, 0x65,
                    0x8f, 0xe2, 0xa9, 0x2f, 0x0e, 0xc2, 0x85, 0x76,
                    0x6b, 0x39, 0xa0, 0xe5, 0x1d, 0xb3, 0x08, 0x62,
                    0xa7, 0xda, 0xc8, 0x60, 0x5f, 0x6a, 0xd6, 0x5d,
                    0xe6, 0xf7, 0x36, 0xf8, 0x18, 0xcd, 0x54, 0xf9
            },

            /*expectedState*/
            {
                    /*v*/
                    {0xd2, 0x16, 0x0d, 0x82, 0xde, 0x92, 0x4d, 0x23,
                            0xae, 0xe3, 0x0c, 0x82, 0x0a, 0x42, 0x23, 0xdd
                    },
                    /*key*/
                    {0xd4, 0x95, 0xda, 0x7a, 0x46, 0x02, 0x36, 0x5e,
                            0xdc, 0xdf, 0xd0, 0x97, 0x6c, 0x60, 0xa8, 0xb6,
                            0x9e, 0x3b, 0xd1, 0x76, 0x25, 0x4e, 0xf8, 0xd2,
                            0x53, 0x7d, 0x95, 0x1b, 0x60, 0x22, 0x92, 0x2f
                    },
                    /*reseedCounter*/
                    1
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                    /*v*/
                    {0xd2, 0x16, 0x0d, 0x82, 0xde, 0x92, 0x4d, 0x23,
                            0xae, 0xe3, 0x0c, 0x82, 0x0a, 0x42, 0x23, 0xdd
                    },
                    /*key*/
                    {0xd4, 0x95, 0xda, 0x7a, 0x46, 0x02, 0x36, 0x5e,
                            0xdc, 0xdf, 0xd0, 0x97, 0x6c, 0x60, 0xa8, 0xb6,
                            0x9e, 0x3b, 0xd1, 0x76, 0x25, 0x4e, 0xf8, 0xd2,
                            0x53, 0x7d, 0x95, 0x1b, 0x60, 0x22, 0x92, 0x2f
                    },
                    /*reseedCounter*/
                    1
            },

            /*setupStrength*/
            CPA_CY_RBG_SEC_STRENGTH_112,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {},

            /*entropyInputLen*/
            48,
            /*entropyInput*/
            {0x5a, 0x4b, 0xec, 0x55, 0x05, 0xfb, 0xda, 0x53,
                    0x46, 0x33, 0x97, 0x39, 0xf1, 0x51, 0x34, 0x3d,
                    0x4f, 0xc2, 0xf1, 0xb3, 0x9c, 0xb9, 0x1c, 0xd1,
                    0x8f, 0x81, 0x49, 0x2b, 0xbe, 0x86, 0x19, 0x44,
                    0xff, 0x1e, 0x49, 0x81, 0xd5, 0x51, 0x53, 0x12,
                    0xf1, 0xa9, 0x9d, 0xc1, 0x6a, 0xc9, 0x5e, 0x8c
            },

            /*expectedState*/
            {
                    /*v*/
                    {0x2d, 0xf3, 0x0a, 0x6e, 0x77, 0x13, 0xd0, 0x4e,
                            0x1d, 0x41, 0xdb, 0x07, 0x8b, 0x5f, 0xf2, 0x6c
                    },
                    /*key*/
                    {0x54, 0xc7, 0xdd, 0xad, 0x36, 0xfc, 0x19, 0xf2,
                            0x29, 0xdf, 0xa5, 0x21, 0x13, 0x6c, 0x02, 0xea,
                            0xf5, 0xd0, 0xdb, 0xfb, 0x33, 0xbb, 0xf6, 0x39,
                            0xd9, 0x58, 0xa8, 0x1c, 0xf9, 0x62, 0x03, 0x38
                    },
                    /*reseedCounter*/
                    1
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                    /*v*/
                    {0xc7, 0xe4, 0xff, 0x1e, 0xf3, 0x55, 0xb3, 0xaf,
                            0x39, 0x69, 0x95, 0xb6, 0x61, 0x15, 0x32, 0x26
                    },
                    /*key*/
                    {0x4b, 0xa7, 0xe1, 0xb9, 0x5f, 0x64, 0x8e, 0x85,
                            0x61, 0xb8, 0x92, 0x76, 0x2d, 0x43, 0x4a, 0xed,
                            0x79, 0xf3, 0x90, 0xd5, 0xda, 0x34, 0x15, 0x1b,
                            0x52, 0xa4, 0xb0, 0x27, 0x7a, 0xba, 0x37, 0xfe
                    },
                    /*reseedCounter*/
                    1
            },

            /*setupStrength*/
            CPA_CY_RBG_SEC_STRENGTH_128,

            /*additionalInputLen*/
            48,
            /*additionalInput*/
            {0x07, 0x4d, 0x58, 0x86, 0x65, 0x8f, 0xf6, 0xb4,
                    0x8e, 0x5d, 0x98, 0x5a, 0xfd, 0xc5, 0x2e, 0x01,
                    0xc9, 0xf2, 0x44, 0x8d, 0x20, 0x70, 0x4f, 0xce,
                    0xf5, 0xa0, 0x27, 0xab, 0xa6, 0x91, 0xe9, 0x73,
                    0x7e, 0x0f, 0x18, 0xaf, 0x5a, 0xdd, 0xe0, 0xd9,
                    0x1f, 0x04, 0x99, 0x85, 0x57, 0xdb, 0xcf, 0xc2
            },

            /*entropyInputLen*/
            48,
            /*entropyInput*/
            {0x59, 0xb0, 0x82, 0xaa, 0x9d, 0xba, 0xc3, 0x54,
                    0x9b, 0xec, 0x0c, 0x8d, 0xa6, 0x10, 0xfd, 0xaf,
                    0x82, 0x72, 0xe7, 0xef, 0xf0, 0x11, 0xd5, 0xd1,
                    0x3d, 0xd4, 0x50, 0x1e, 0x1a, 0x6e, 0x99, 0xfb,
                    0xf7, 0x7d, 0x87, 0x4e, 0xeb, 0x5b, 0xf4, 0x0a,
                    0xd9, 0xb1, 0x95, 0xfd, 0x1e, 0x82, 0x0c, 0xe0
            },

            /*expectedState*/
            {
                    /*v*/
                    {0x9f, 0x65, 0xa8, 0xc0, 0x75, 0x13, 0xca, 0xdd,
                            0x49, 0x68, 0x09, 0xc3, 0x4a, 0x94, 0xdb, 0x7d
                    },
                    /*key*/
                    {0xb8, 0xf2, 0x1f, 0xcd, 0xc1, 0x8f, 0xd9, 0x90,
                            0xe9, 0x70, 0x34, 0xf2, 0x73, 0xc6, 0x05, 0x56,
                            0xfa, 0xfc, 0xc8, 0xac, 0x26, 0xd1, 0xf2, 0x95,
                            0xb6, 0x95, 0x3a, 0x7e, 0x32, 0x25, 0xc1, 0x1d
                    },
                    /*reseedCounter*/
                    1
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                    /*v*/
                    {0x9f, 0x65, 0xa8, 0xc0, 0x75, 0x13, 0xca, 0xdd,
                            0x49, 0x68, 0x09, 0xc3, 0x4a, 0x94, 0xdb, 0x7d
                    },
                    /*key*/
                    {0xb8, 0xf2, 0x1f, 0xcd, 0xc1, 0x8f, 0xd9, 0x90,
                            0xe9, 0x70, 0x34, 0xf2, 0x73, 0xc6, 0x05, 0x56,
                            0xfa, 0xfc, 0xc8, 0xac, 0x26, 0xd1, 0xf2, 0x95,
                            0xb6, 0x95, 0x3a, 0x7e, 0x32, 0x25, 0xc1, 0x1d
                    },
                    /*reseedCounter*/
                    1
            },

            /*setupStrength*/
            CPA_CY_RBG_SEC_STRENGTH_128,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {},

            /*entropyInputLen*/
            48,
            /*entropyInput*/
            {0xdd, 0xc0, 0x58, 0xfd, 0xd3, 0xf7, 0x82, 0xc5,
                    0xea, 0xd1, 0xc8, 0x41, 0x37, 0x62, 0x09, 0xb8,
                    0x74, 0x62, 0xd6, 0x63, 0x4d, 0xfe, 0x48, 0x17,
                    0xb7, 0xdb, 0x10, 0xb7, 0x3a, 0x6d, 0xf8, 0xda,
                    0xe8, 0x9a, 0x8c, 0x30, 0x13, 0x6e, 0x12, 0x16,
                    0xc8, 0x74, 0x37, 0x49, 0x0a, 0xd2, 0x78, 0xbc
            },

            /*expectedState*/
            {
                    /*v*/
                    {0xa8, 0x7f, 0x9e, 0xf7, 0xd4, 0xee, 0x59, 0x9e,
                            0xdf, 0x2e, 0x54, 0xa6, 0xeb, 0x78, 0xa4, 0xb7
                    },
                    /*key*/
                    {0x3e, 0x3e, 0xf1, 0xbe, 0xe1, 0x7f, 0x2d, 0xa2,
                            0x52, 0x75, 0xd7, 0xef, 0x79, 0x01, 0x93, 0x17,
                            0x98, 0x2c, 0xb7, 0xb7, 0x32, 0x10, 0x95, 0xbd,
                            0xd1, 0xd6, 0xec, 0xe6, 0x00, 0xb9, 0xfc, 0x11
                    },
                    /*reseedCounter*/
                    1
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                    /*v*/
                    {0x15, 0x46, 0xe1, 0x07, 0xeb, 0xd7, 0x7d, 0x56,
                            0x1d, 0x21, 0x4f, 0x75, 0x8a, 0x17, 0x1e, 0x33
                    },
                    /*key*/
                    {0x7a, 0xa4, 0xc1, 0xb0, 0x80, 0xc7, 0x5a, 0x2c,
                            0x59, 0x1e, 0x66, 0xf1, 0xad, 0x80, 0x6a, 0xda,
                            0x49, 0xf1, 0x49, 0x4c, 0xdf, 0x1f, 0xeb, 0xac,
                            0xd7, 0xa7, 0x2a, 0xbf, 0xd0, 0x25, 0x30, 0xe9
                    },
                    /*reseedCounter*/
                    1
            },

            /*setupStrength*/
            CPA_CY_RBG_SEC_STRENGTH_192,

            /*additionalInputLen*/
            48,
            /*additionalInput*/
            {0x2b, 0x63, 0xed, 0x18, 0x6b, 0x14, 0x27, 0x90,
                    0x5d, 0x04, 0x54, 0x26, 0xcd, 0xd1, 0x13, 0x4b,
                    0xd2, 0x0d, 0xde, 0x1f, 0xba, 0x05, 0xee, 0x12,
                    0x27, 0x43, 0x44, 0x09, 0xb0, 0x98, 0x8e, 0x2e,
                    0xd0, 0x6c, 0x37, 0x5a, 0x52, 0x70, 0xc5, 0xc4,
                    0x1c, 0x3c, 0x80, 0xbb, 0x4c, 0x60, 0xf3, 0xa2
            },

            /*entropyInputLen*/
            48,
            /*entropyInput*/
            {0x0b, 0xa3, 0x18, 0x0f, 0xad, 0xf0, 0xda, 0xd6,
                    0x21, 0x09, 0xf7, 0x75, 0x54, 0x84, 0xf2, 0x16,
                    0x27, 0x94, 0x12, 0xbc, 0x6e, 0xdc, 0x02, 0x6d,
                    0x1f, 0xce, 0xb4, 0xff, 0xd3, 0xff, 0x35, 0x69,
                    0xb4, 0x81, 0x0b, 0xe8, 0x32, 0xb2, 0x61, 0xff,
                    0x06, 0x35, 0x5d, 0xa7, 0x6f, 0xee, 0xd9, 0xd8
            },

            /*expectedState*/
            {
                    /*v*/
                    {0x0a, 0x5c, 0x97, 0xec, 0x3e, 0x16, 0xe7, 0x0e,
                            0xd1, 0xf1, 0x33, 0x78, 0x9b, 0xda, 0xd9, 0x51
                    },
                    /*key*/
                    {0xe4, 0x3a, 0xa7, 0xc4, 0x8b, 0x12, 0x50, 0x43,
                            0xb2, 0x22, 0x01, 0x00, 0x98, 0x0b, 0xf5, 0x2e,
                            0xb1, 0x1e, 0x65, 0x48, 0x57, 0x99, 0xba, 0xe8,
                            0x59, 0x7b, 0x97, 0xf3, 0x61, 0xe3, 0xfb, 0x6d
                    },
                    /*reseedCounter*/
                    1
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_TRUE,
            /*initialState*/
            {
                    /*v*/
                    {0x0a, 0x5c, 0x97, 0xec, 0x3e, 0x16, 0xe7, 0x0e,
                            0xd1, 0xf1, 0x33, 0x78, 0x9b, 0xda, 0xd9, 0x51
                    },
                    /*key*/
                    {0xe4, 0x3a, 0xa7, 0xc4, 0x8b, 0x12, 0x50, 0x43,
                            0xb2, 0x22, 0x01, 0x00, 0x98, 0x0b, 0xf5, 0x2e,
                            0xb1, 0x1e, 0x65, 0x48, 0x57, 0x99, 0xba, 0xe8,
                            0x59, 0x7b, 0x97, 0xf3, 0x61, 0xe3, 0xfb, 0x6d
                    },
                    /*reseedCounter*/
                    1
            },

            /*setupStrength*/
            CPA_CY_RBG_SEC_STRENGTH_192,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {},

            /*entropyInputLen*/
            48,
            /*entropyInput*/
            {0xc7, 0x73, 0x9a, 0x84, 0x2d, 0x3e, 0x3d, 0x5d,
                    0x46, 0xcb, 0x6d, 0x1e, 0x59, 0x0f, 0x3a, 0x2e,
                    0xa0, 0x45, 0x05, 0xcb, 0xd3, 0xc1, 0x01, 0x23,
                    0x03, 0x84, 0x24, 0xaf, 0x06, 0x77, 0xf7, 0xff,
                    0x38, 0x95, 0x1a, 0x3f, 0xbc, 0x75, 0x94, 0x73,
                    0x4b, 0x02, 0x12, 0x08, 0x94, 0x9a, 0x3a, 0x2a
            },

            /*expectedState*/
            {
                    /*v*/
                    {0x2f, 0xea, 0x0a, 0x8a, 0x11, 0x06, 0xc5, 0x97,
                            0xdf, 0x94, 0x00, 0x64, 0xdf, 0x93, 0x34, 0xd3
                    },
                    /*key*/
                    {0x59, 0xae, 0x34, 0x4b, 0xe0, 0x75, 0x3b, 0xec,
                            0x2f, 0x9b, 0xb9, 0xcb, 0x1b, 0x22, 0xf3, 0xfa,
                            0x35, 0x61, 0xd4, 0x3c, 0xf8, 0x46, 0x67, 0x1b,
                            0x4f, 0x32, 0x7b, 0xb2, 0x79, 0x09, 0x18, 0x8e
                    },
                    /*reseedCounter*/
                    1
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                    /*v*/
                    {0x22, 0xad, 0x87, 0xff, 0x7f, 0x61, 0xa8, 0x22,
                            0x17, 0x39, 0xe1, 0x35, 0x46, 0x99, 0x78, 0x61
                    },
                    /*key*/
                    {0x52, 0x5a, 0xba, 0x6c, 0xe2, 0xee, 0xd0, 0x0f,
                            0xa9, 0x90, 0x05, 0x35, 0x57, 0x85, 0x1a, 0x3f,
                            0x06, 0x06, 0x6f, 0x16, 0x43, 0x20, 0x10, 0x6c,
                            0x5f, 0x5b, 0xc3, 0x42, 0x69, 0x5b, 0xad, 0xc1
                    },
                    /*reseedCounter*/
                    1
            },

            /*setupStrength*/
            CPA_CY_RBG_SEC_STRENGTH_256,

            /*additionalInputLen*/
            48,
            /*additionalInput*/
            {0x86, 0xea, 0x77, 0x31, 0xae, 0xa2, 0x62, 0x37,
                    0x47, 0xd7, 0x96, 0xc9, 0x87, 0xc4, 0xef, 0xe8,
                    0x3c, 0xa4, 0x56, 0x51, 0xd7, 0xf4, 0x1c, 0xf7,
                    0xa3, 0x5b, 0xad, 0x1b, 0x0f, 0xa7, 0x7e, 0x76,
                    0x41, 0x0a, 0x05, 0x5d, 0xb0, 0x4f, 0xef, 0x86,
                    0xe0, 0x85, 0x2b, 0x3b, 0xd2, 0xf8, 0x51, 0x32
            },

            /*entropyInputLen*/
            48,
            /*entropyInput*/
            {0x09, 0x2b, 0x34, 0x02, 0x14, 0xaa, 0x50, 0xf1,
                    0x54, 0x53, 0x95, 0x73, 0x34, 0x77, 0xd4, 0x84,
                    0xdb, 0x10, 0x40, 0x66, 0xda, 0x31, 0x48, 0xda,
                    0x57, 0x57, 0xde, 0x9f, 0x48, 0x2b, 0x6f, 0x07,
                    0x49, 0xd4, 0xd0, 0x7a, 0xe0, 0xe8, 0x8f, 0x47,
                    0x49, 0xaf, 0xa9, 0xa3, 0x60, 0x29, 0xba, 0x88
            },

            /*expectedState*/
            {
                    /*v*/
                    {0xf8, 0x28, 0xa6, 0x91, 0x58, 0x06, 0x50, 0x33,
                            0x27, 0x41, 0xf9, 0xdc, 0x19, 0xd8, 0x5c, 0x12
                    },
                    /*key*/
                    {0xf0, 0xe4, 0xd0, 0x12, 0x5a, 0xe8, 0x58, 0x8f,
                            0x43, 0xe5, 0x52, 0x60, 0x9f, 0xa6, 0xaa, 0xea,
                            0x7e, 0x72, 0xfd, 0x7f, 0x76, 0xfd, 0xd2, 0xf9,
                            0xfb, 0x2c, 0xd0, 0x67, 0xad, 0x82, 0x36, 0xed
                    },
                    /*reseedCounter*/
                    1
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        },
        {
            /*isDFUsed*/
            CPA_FALSE,
            /*predictionResistanceEnabled*/
            CPA_FALSE,
            /*initialState*/
            {
                    /*v*/
                    {0xf8, 0x28, 0xa6, 0x91, 0x58, 0x06, 0x50, 0x33,
                            0x27, 0x41, 0xf9, 0xdc, 0x19, 0xd8, 0x5c, 0x12
                    },
                    /*key*/
                    {0xf0, 0xe4, 0xd0, 0x12, 0x5a, 0xe8, 0x58, 0x8f,
                            0x43, 0xe5, 0x52, 0x60, 0x9f, 0xa6, 0xaa, 0xea,
                            0x7e, 0x72, 0xfd, 0x7f, 0x76, 0xfd, 0xd2, 0xf9,
                            0xfb, 0x2c, 0xd0, 0x67, 0xad, 0x82, 0x36, 0xed
                    },
                    /*reseedCounter*/
                    1
            },

            /*setupStrength*/
            CPA_CY_RBG_SEC_STRENGTH_256,

            /*additionalInputLen*/
            0,
            /*additionalInput*/
            {},

            /*entropyInputLen*/
            48,
            /*entropyInput*/
            {0x4b, 0x0d, 0xf0, 0xc6, 0x29, 0xa8, 0x0f, 0x55,
                    0xd5, 0x6a, 0x80, 0x38, 0xf9, 0xee, 0x4c, 0xbe,
                    0x36, 0xda, 0x81, 0xa3, 0xdc, 0x01, 0x65, 0xbb,
                    0x95, 0x7e, 0xe8, 0x7b, 0x48, 0x2b, 0x87, 0x88,
                    0x3e, 0x7b, 0xa5, 0x2c, 0x3a, 0x02, 0x79, 0xec,
                    0xc2, 0xe9, 0x7b, 0x25, 0xea, 0x45, 0x88, 0x6f
            },

            /*expectedState*/
            {
                    /*v*/
                    {0xae, 0x94, 0x2c, 0xd4, 0xf4, 0x95, 0x55, 0x12,
                            0x19, 0x13, 0xe6, 0x1b, 0xeb, 0x80, 0x4c, 0xf4
                    },
                    /*key*/
                    {0x6c, 0x41, 0xf6, 0xca, 0x37, 0x57, 0x5e, 0xf8,
                            0x09, 0xba, 0x77, 0xf4, 0xfc, 0x69, 0x36, 0x07,
                            0xb0, 0x50, 0x4f, 0xfe, 0x14, 0x2a, 0x3e, 0x28,
                            0x2c, 0x2c, 0x2f, 0x84, 0x30, 0xc1, 0x7e, 0x50
                    },
                    /*reseedCounter*/
                    1
            },
            /*expectedStatus*/
            CPA_STATUS_SUCCESS
        }
};

#define LAC_DRBG_HT_NUM_INSTANTIATE_VECTORS \
    (sizeof(instantiateVectors) / sizeof(lac_drbg_ht_instantiate_vector_t))

#define LAC_DRBG_HT_NUM_GENERATE_DF_VECTORS \
    (sizeof(generateDFVectors) / sizeof(lac_drbg_ht_generate_vector_t))

#define LAC_DRBG_HT_NUM_GENERATE_NODF_VECTORS \
    (sizeof(generateNoDFVectors) / sizeof(lac_drbg_ht_generate_vector_t))

#define LAC_DRBG_HT_NUM_RESEED_DF_VECTORS \
    (sizeof(reseedDFVectors) / sizeof(lac_drbg_ht_reseed_vector_t))

#define LAC_DRBG_HT_NUM_RESEED_NODF_VECTORS \
    (sizeof(reseedNoDFVectors) / sizeof(lac_drbg_ht_reseed_vector_t))


#endif
